﻿/*
  bbs2chreader であぼーん登録を行うと document オブジェクトで b2raboneadd イベントが発生する
  イベントハンドラの aEvent 引数から、あぼーんされた文字列とそのタイプが取得できる
    aEvent.detail : あぼーんタイプ( b2rIAboneManager.ABONE_TYPE_XXX )
    aEvent.sourceEvent.type : あぼーんされた文字列
*/

/**
 * @fileOverview 即時あぼーんを扱います。
 */
/**
 * bbs2chreader が発生させる b2raboneadd イベントを捕捉して、あぼーんの即時適用に対応します。
 * @static
 */
var b2rAboneHandler = {
	/** イベントリスナを登録します。*/
	startup: function(){
		document.addEventListener("b2raboneadd", b2rAboneHandler, false);
	},
	/** レスの内容をすべてあぼーんに設定します。
	 * @param {element} node あぼーんするレスの親要素
	 */
	markAsAbone: function(node) {
		var names = ["resName", "resMail", "resDate", "resBody"];
		for (var i = 0; i < names.length; i++) {
			var child = this.getNode(node, names[i]);
			child.textContent = "ABONE";
		}
	},
	/** 親要素から指定されたクラス名を持つ要素を取得します。
	 * @param  {element} parentNode 親要素
	 * @param  {element} className クラス名
	 * @return {element} マッチした要素
	 */
	getNode: function(parentNode, className){
		var xpathResult = document.evaluate("descendant::*[starts-with(@class,'" + className + "')]", parentNode, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		return xpathResult.singleNodeValue;
	},
	/** b2raboneadd イベントを処理します。
	 * @param {event} aEvent イベント
	 */
	handleEvent: function(aEvent){
		var word = aEvent.sourceEvent.type;
		var type = aEvent.detail;
		var className = ["resName", "resMail", "resID", "resBody"][type];
		var resItems = ResNodes.getContainers();
		for (var i = 0; i < resItems.length; i++){
			var node = resItems.items(i);
			var childNode = this.getNode(node, className);
			if (childNode.textContent.indexOf(word) != -1)
				this.markAsAbone(node);
		}
	}
};