﻿// general.js
// (2007/02/03)

/**
 * @fileOverview メインのスクリプト。ゴタゴタしている。
 */

// excerpt from prototype.js 1.4.1 (but slightly modified)
// Copyright (c) 2005-2007 Sam Stephenson
/** prototype.js の bind() です。this を束縛します。 */
Function.prototype.bind = function() {
	var self = this;
	var args = Array.prototype.slice.call(arguments);
	var object = args.shift();
  	return function() {
    	return self.apply(object, args.concat(Array.prototype.slice.call(arguments)));
  	}
}

/**
 * DOM 操作のために、スキンの静的なノードを格納して管理します。
 * @static
 */
var Nodes = {
	/** ヘッダ
	 * @type element
	 */
	header:     document.getElementById("header"),
	/** ヘッダの右側のコマンドバー
	 * @type element
	 */
	command:    document.getElementById("command"),
	/** 検索/抽出ボックス
	 * @type element
	 */
	findBox:    document.getElementById("findbox"),
	/** 検索/抽出ボックスのテキストボックス
	 * @type element
	 */
	findBoxText:document.getElementById("findboxText"),
	/** レス数
	 * @type element
	 */
	resCount:   document.getElementById("resCount"),
	/** ステータス表示
	 * @type element
	 */
	statusText: document.getElementById("statusText"),
	/** スレッドタイトル
	 * @type element
	 */
	threadName: document.getElementById("threadName"),
	/** 板の名称
	 * @type element
	 */
	boardName:  document.getElementById("boardName"),
	/** サイドバー<strong>（暫定）</strong>
	 * @type element
	 */
	sidePanel:  document.getElementById("sidePanel"),
	/** スクロールビュー<strong>（暫定）</strong>
	 * @type element
	 */
	scrollView: document.getElementById("scrollView"),
	/** 既読レスと新着レスとの境界
	 * @type element
	 */
	newMark:    document.getElementById("NewMark"),
	/** レスのコンテナ
	 * @type element
	 */
	content:    document.getElementById("content"),
	/** 各メンバを初期化します。*/
	initialise: function() {
		this.header      = document.getElementById("header");
		this.command     = document.getElementById("command");
		this.menu        = document.getElementById("menu");
		this.findBox     = document.getElementById("findbox");
		this.findBoxText = document.getElementById("findboxText");
		this.resCount    = document.getElementById("resCount");
		this.statusText  = document.getElementById("statusText");
		this.threadName  = document.getElementById("threadName");
		this.boardName   = document.getElementById("boardName");
		this.sidePanel   = document.getElementById("sidePanel");
		this.scrollView  = document.getElementById("scrollView");
		this.newMark     = document.getElementById("NewMark");
		this.content     = document.getElementById("content");
	},
	/** レス要素の配列を取得します。<em>(obsolete)</em>
	 * @return {Array} 配列
	 */
	getResItems: function() {
		return document.getElementsByName("resItems");
	},
	/** ID 要素の配列を取得します。<em>(obsolete)</em>
	 * @return {Array} 配列
	 */
	getIDItems: function() {
		return document.getElementsByName("idItems");
	},
	/** 指定のレス番号のレス要素を取得します。
	 * @param  {Number} index レス番号
	 * @return {element} 要素
	 */
	getRes: function(index) {
		return document.getElementById("res" + index);
	},
	/** 指定のレス番号の本文要素を取得します。
	 * @param  {Number} index レス番号
	 * @return {element} 要素
	 */
	getBody: function(index) {
		return document.getElementById("body" + index);
	},
	/** 指定のレス番号の名前欄要素を取得します。
	 * @param  {Number} index レス番号
	 * @return {element} 要素
	 */
	getName: function(index) {
		return document.getElementById("res" + index).childNodes[1].childNodes[3].childNodes[1];
	},
	/** 指定のレス番号のメール欄要素を取得します。
	 * @param  {Number} index レス番号
	 * @return {element} 要素
	 */
	getMail: function(index) {
		return document.getElementById("res" + index).childNodes[1].childNodes[3].childNodes[3];
	},
	/** 指定のレス番号の日付欄要素を取得します。
	 * @param  {Number} index レス番号
	 * @return {element} 要素
	 */
	getDate: function(index) {
		return document.getElementById("res" + index).childNodes[1].childNodes[3].childNodes[5];
	},
	/** 指定のレス番号の ID 欄要素を取得します。
	 * @param  {Number} index レス番号
	 * @return {element} 要素
	 */
	getID: function(index) {
		return document.getElementById("res" + index).childNodes[1].childNodes[3].childNodes[7];
	},
	/** 指定のレス番号の BeID 欄要素を取得します。
	 * @param  {Number} index レス番号
	 * @return {element} 要素
	 */
	getBeID: function(index) {
		return document.getElementById("res" + index).childNodes[1].childNodes[3].childNodes[9];
	},
	/** 指定のレス要素から、レス番号を取得します。
	 * @param  {element} node 要素
	 * @return {Number}  レス番号
	 */
	getIndexFromRes: function(node) {
		if (node) return parseInt(node.id.slice(3));
	},
	/** 指定の本文要素から、レス番号を取得します。
	 * @param  {element} node 要素
	 * @return {Number}  レス番号
	 */
	getIndexFromBody: function(node) {
		if (node) return parseInt(node.id.slice(4));
	}
};

// XPathItem
/**
 * XPath 式を評価し、その結果を保持します。
 * @class
 * @param {String}  xPath         XPath 式
 * @param {element} [contextNode] 評価する親ノード
 * @property {Node} snapshot      スナップショット
 * @property {Node} length        マッチしたノードの数
 * @property {Node} first         最初のノード
 * @property {Node} last          最後のノード
 */
function XPathItem() {
	this.initialise.apply(this, arguments);
}
XPathItem.prototype = {
	/** XPath 式 
	 * @type String
	 */
	xPath:       null,
	/** 評価する親ノード
	 * @type element
	 */
	contextNode: null,
	/** 評価結果 
	 * @type XPathResult
	 */
	result:    null,
	/** コンストラクタから自動的に呼ばれ、初期化処理を行います。 */
	initialise: function(xPath, contextNode) {
		this.xPath = xPath;
		this.contextNode = (contextNode) ? contextNode : document;
		this.result = document.evaluate(this.xPath, this.contextNode, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
	},
	get snapshot() {
		return this.result.snapshotItem;
	},
	/** スナップショットから、指定のインデックスのノードを取得します。
	 * @param  {Number}  index インデックス
	 * @return {element} 要素
	 */
	items: function(index) {
		return this.result.snapshotItem(index);
	},
	get length() {
		return this.result.snapshotLength;
	},
	get first() {
		if (this.result.snapshotLength > 0) return this.result.snapshotItem(0);
	},
	get last() {
		if (this.result.snapshotLength > 0) return this.result.snapshotItem(this.result.snapshotLength - 1);
	}
}

/**
 * DOM 操作のために、主に動的なノードを管理します。
 * @static
 */
var ResNodes = {
	/** コンテキストメニューが表示されているかどうかを取得します。
	 * @return {Boolean} 表示されていれば true
	 */
	isContextMenuVisible: function() {
		var result = document.evaluate(".//div[@class='contextMenu']", document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		return result.singleNodeValue ? true : false;
	},
	/** レス要素の子要素から、親要素を検索します。
	 * @param  {element} contextNode 評価する親ノード
	 * @param  {Boolean} [popup]     ポップアップ内の要素を含めるかどうか
	 * @return {element} 要素
	 */
	getParentContainer: function(contextNode, popup) {
		var result = document.evaluate(popup ? ".//ancestor::dl" : ".//ancestor::dl[@id][@class='resContainer']", contextNode, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		return result.singleNodeValue;
	},
	/** 指定されたレス要素から、レス番号を取得します。
	 * @param  {element} node 要素
	 * @return {Number}  レス番号
	 */
	getIndexByContainer: function(node) {
		if (node) return parseInt(node.id.slice(3));
	},
	/** レス要素の親要素である、レスセレクタ要素からレス番号を取得します。
	 * @param  {element} node 要素
	 * @return {Number}  レス番号
	 */
	getIndexBySelector: function(node) {
		//if (node) return parseInt(node.firstChild.id.slice(3));
		if (node) return parseInt(node.firstChild.id.slice(3));
	},
	/** レス要素の子要素である、ID 要素からレス番号を取得します。
	 * @param  {element} node 要素
	 * @return {Number}  レス番号
	 */
	getIndexByID: function(node) {
		if (node) return parseInt(node.parentNode.parentNode.parentNode.id.slice(3));
	},
	/** レス要素の子要素である、本文要素からレス番号を取得します。
	 * @param  {element} node 要素
	 * @return {Number}  レス番号
	 */
	getIndexByBody: function(node) {
		if (node) return parseInt(node.id.slice(4));
	},
	/** 指定されたレス番号から、レス要素を取得します。
	 * @param  {Number}  index レス番号
	 * @param  {element} contextNode 評価する親ノード
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getContainerByIndex: function(index, contextNode) {
		return new XPathItem(".//dl[@id='res" + index +"']", contextNode).first;
	},
	/** 指定されたレス番号から、本文要素を取得します。
	 * @param  {Number}  index レス番号
	 * @param  {element} contextNode 評価する親ノード
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getBodyByIndex: function(index, contextNode) {
		return new XPathItem(".//dd[@id='body" + index +"']", contextNode).first;
	},
	/** すべてのレス要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getContainers: function(contextNode, popup) {
		return new XPathItem(popup ? ".//dl" : ".//dl[@id][@class='resContainer']", contextNode);
	},
	/** すべての既読レスヘッダ要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getHeaders: function(contextNode) {
		return new XPathItem(".//dt[@class='resHeader']", contextNode);
	},
	/** すべての新着レスヘッダ要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getNewHeaders: function(contextNode) {
		return new XPathItem(".//dt[@class='resNewHeader']", contextNode);
	},
	/** すべてのレス番号要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getNumbers: function(contextNode, popup) {
		return new XPathItem(popup ? ".//descendant::div[@class='resNumber']/a" : ".//dl[@id][@class='resContainer']/descendant::div[@class='resNumber']/a", contextNode);
	},
	/** すべての非マーク済みレス番号要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getUnmarkedNumbers: function(contextNode, popup) {
		return new XPathItem(popup ? ".//descendant::div[@class='resNumber']/a[not(@class)]" : ".//dl[@id][@class='resContainer']/descendant::div[@class='resNumber']/a[not(@class)]", contextNode);
	},
	/** すべての名前欄要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getNames: function(contextNode, popup) {
		return new XPathItem(popup ? ".//descendant::span[@class='resName']" : ".//dl[@id][@class='resContainer']/descendant::span[@class='resName']", contextNode);
	},
	/** すべてのメール欄要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getMails: function(contextNode, popup) {
		return new XPathItem(popup ? ".//descendant::span[@class='resMail']" : ".//dl[@id][@class='resContainer']/descendant::span[@class='resMail']", contextNode);
	},
	/** すべての日付欄要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getDates: function(contextNode, popup) {
		return new XPathItem(popup ? ".//descendant::span[@class='resDate']" : ".//dl[@id][@class='resContainer']/descendant::span[@class='resDate']", contextNode);
	},
	/** すべての ID 欄要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getIDs: function(contextNode, popup) {
		return new XPathItem(popup ? ".//descendant::span[@rel]" : ".//dl[@id][@class='resContainer']/descendant::span[@rel]", contextNode);
	},
	/** ID として有効な、すべての ID 欄要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getValidIDs: function(contextNode, popup) {
		return new XPathItem(".//dl[@id][@class='resContainer']/descendant::span[string-length(string(@rel)) >= 8]", contextNode);
	},
	/** すべての BeID 欄要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getBeIDs: function(contextNode, popup) {
		return new XPathItem(".//dl[@id][@class='resContainer']/descendant::span[@class='resBeID']", contextNode);
	},
	/** すべての本文要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getBodies: function(contextNode, popup) {
		return new XPathItem(".//dl[@id][@class='resContainer']/descendant::dd[@class='resBody']", contextNode);
	},
	/** すべてのレスアンカー要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getResAnchors: function(contextNode, popup) {
		return new XPathItem(".//dd[@id]/descendant::a[@class='resPointer']", contextNode);
	},
	/** すべての ID アンカー要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getIDAnchors: function(contextNode, popup) {
		return new XPathItem(".//dd[@class='resBody']/descendant::span[starts-with(@class, 'mesID_')]", contextNode);
	},
	/** すべての外部リンク要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getOutLinks: function(contextNode, popup) {
		return new XPathItem(".//dd[@class='resBody']/descendant::a[@class='outLink']", contextNode);
	},
	/** すべてのレスセレクタ要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getSelectors: function(contextNode) {
		var obj = new XPathItem(".//div[@class='resUnselected']|.//div[@class='resSelected']", contextNode);
		obj.selected   = new XPathItem(".//div[@class='resSelected']", contextNode);
		obj.unselected = new XPathItem(".//div[@class='resUnselected']", contextNode);
		return obj;
	},
	/** Favicon の link 要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getFavicon: function(contextNode) {
		return new XPathItem(".//link[@rel='icon']", contextNode).first;
	},
	/** すべてのレス要素の親要素である div 要素を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @return {XPathItem} XPathItem オブジェクト
	 */
	getContent: function(contextNode) {
		return new XPathItem(".//div[@id='content']", contextNode).first;
	},
	/** レス番号から、指定された形式でレスの内容を取得。
	 * @param  {Number} index  レス番号
	 * @param  {String} format Jane 形式で取得する場合は "Jane" を指定する（デフォルトで 2ch 形式）
	 * @return {String} 文字列
	 */
	getEntireTextByIndex: function(index, format) {
		var container = this.getContainerByIndex(index);
		if (!container) return null;
		container = container.parentNode;
		var number = index;
		var name = this.getNameText(container);
		var mail = this.getMailText(container);
		var date = this.getDateText(container);
		var id   = this.getIDText(container);
		var beid = this.getBeIDText(container);
		var body = this.getBodyText(container);
		var str = "";
		if (format == "Jane") {
			str = number + " 名前：" + name + "[" + mail + "]" + " 投稿日：" + date;
		} else {
			str = number + " ：" + name + "：" + date;
		}
		if (id)   str += " " + id;
		if (beid) str += " " + beid;
		str += "\n" + body;
		return str;
	},
	/** 名前欄の文字列を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {String} 文字列
	 */
	getNameText: function(contextNode, popup) {
		return this.getNames(contextNode, popup).first.textContent;
	},
	/** メール欄の文字列を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {String} 文字列
	 */
	getMailText: function(contextNode, popup) {
		return this.getMails(contextNode, popup).first.textContent.replace(/^ | $/g, "");
	},
	/** 日付欄の文字列を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {String} 文字列
	 */
	getDateText: function(contextNode, popup) {
		return this.getDates(contextNode, popup).first.textContent.replace(/^ | $/g, "");
	},
	/** ID 欄の文字列を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {String} 文字列
	 */
	getIDText: function(contextNode, popup) {
		return this.getIDs(contextNode, popup).first.textContent.replace(/^ | $/g, "");
	},
	/** BeID 欄の文字列を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {String} 文字列
	 */
	getBeIDText: function(contextNode, popup) {
		return this.getBeIDs(contextNode, popup).first.textContent.replace(/^ | $/g, "");
	},
	/** 本文の文字列を取得します。
	 * @param  {element} contextNode  評価する親ノード
	 * @param  {Boolean} [popup]      ポップアップ内の要素を含めるかどうか
	 * @return {String} 文字列
	 */
	getBodyText: function(contextNode, popup) {
		return ThreadDocument.getInnerText(this.getBodies(contextNode, popup).first).replace(/ \n /g, "\n").replace(/^ | $/g, "");
	}
};

// ThreadDocument
// いろいろ
/**
 * ドキュメント全体を管理します。
 * @static
 */
var ThreadDocument = {
	/** スレッドタイトル
	 * @type String
	 */
	title: "",
	/** ステータス文字列
	 * @type String
	 */
	status: "",
	/** すべてのレスの件数
	 * @type Number
	 */
	countAll: null,
	/** 既読レスの件数
	 * @type Number
	 */
	countRead: null,
	/** 新着レスの件数
	 * @type Number
	 */
	countUnread: null,
	/** サーバのホスト名
	 * @type String
	 */
	host: "",
	/** 板の ID
	 * @type String
	 */
	boardName: "",
	/** スレッド番号
	 * @type String
	 */
	threadID: "",
	/** スレッド表示のオプション文字列
	 * @type String
	 */
	option: "",
	/** 最後に再読み込みを行った日付
	 * @type Date
	 */
	date: new Date(),
	/** Footer.html から呼ばれる関数で、このスキン全体のスクリプトをここで初期化します。
	 * @param {String} statusText  &gt;STATUS/&lt; タグで置換された文字列
	 * @param {String} getResCount &gt;GETRESCOUNT/&lt; タグで置換された文字列
	 * @param {String} newResCount &gt;NEWRESCOUNT/&lt; タグで置換された文字列
	 * @param {String} allResCount &gt;ALLRESCOUNT/&lt; タグで置換された文字列
	 */
	initialise: function(statusText, getResCount, newResCount, allResCount) {
		if (location.href.match(/lite=true/)) return;
		this.title          = document.title;
		this.status         = statusText;
		this.countRead      = parseInt(getResCount);
		this.countUnread    = parseInt(newResCount);
		this.countAll       = parseInt(allResCount);
		this.host           = EXACT_URL.replace(/http:\/\/(.+?)\/test\/read\.cgi\/.*/, "$1");
		//if (location.href.match(/read\.cgi\/(.*?)\/(.*?)\/(.*)/)) {
		if (location.href.match(/read\.cgi\/(.*?)\/(.*)\/(.*)/)) {
			this.boardName = RegExp.$1;
			this.threadID  = RegExp.$2;
			this.option    = RegExp.$3;
		}
		Nodes.initialise();
		if (location.href.match(/(2ch\.net|bbspink\.com|machi\.to)/)) Nodes.boardName.textContent = boardTable[this.boardName] ? boardTable[this.boardName] : "";
		PopupPreventer.initialise();
		ResPopup.initialise();
		IDPopup.initialise();
		TrackbackPopup.initialise();
		ImagePopup.initialise();
		UrlPopup.initialise();
		VideoPopup.initialise();
		Osusume2chPopup.initialise();
		RelatedKeywordsPopup.initialise();
		IDExtract.initialise();
		KeyInput.initialise();
		AutoReload.initialise();
		HistoryManager.initialise();
		PageScroller.initialise();
		MultipleResSelector.initialise();
		b2rAboneHandler.startup();
		FxFindHandler.initialise();
		NumericScroller.initialise();
		this.setStatus(true);
		if (!Bookmark.initialise()) this.scrollToNewRes();
		
		window.setTimeout((function() {
			this.modifyAnchors();
			this.colourIDs();
			this.markTrackbackedResNumbers();
		}).bind(this), 1);

		if (SkinPref.getBool("enableContract", true)) window.addEventListener("resize", this.contractCaption.bind(this), false);
		window.addEventListener("contextmenu", ThreadDocument.onContextMenu, false);
	},
	/** contextmenu イベントを処理します。
	 * @param {event} e イベント
	 */
	onContextMenu: function(e) {
		if (e.target.tagName != "SPAN") if (e.target.tagName != "A") return;
		//if (window.getSelection().toString().length > 0) return;
		var selection = window.getSelection();
		selection.removeAllRanges();
		if (e.target.textContent.match(/ID:([^\(\)]{8,})/)) {
			var range = document.createRange();
			range.setStart(e.target.firstChild, 3);
			range.setEnd(e.target.firstChild, 3 + RegExp.$1.length);
			selection.addRange(range);
		} else {
			window.getSelection().selectAllChildren(e.target);
		}
		//e.returnValue = true;
	},
	/** 文字列をクリップボードにコピーします。
	 * @param {String} text 文字列
	 */
	setClipboard: function(text) {
		// 最速インターフェース研究会より:
		// http://la.ma.la/misc/js/setClip board2.html
		swf_data = SKIN_PATH + "setClipboard.swf";
		var tmp = document.createElement("div");
		tmp.innerHTML = [
			 '<embed src="', swf_data, '"'
			,' FlashVars="code=',encodeURIComponent(text),'"'
			,' width="0" height="0"'
			,'></embed>'
		].join("");
		with(tmp.style){
			position ="absolute";
			left = "-10px";
			top  = "-10px";
			visibility = "hidden";
		};
		document.body.appendChild(tmp);
		setTimeout(function(){document.body.removeChild(tmp)},1000)
		return;
	},
	/** 指定されたノードの文字列を、改行を含めて正しく取得します。
	 * @param  {element} node ノード
	 * @return {String} 文字列
	 */
	getInnerText: function(node) {
	   var nodes = node.childNodes,
	       ret   = [];
	   for (var i = 0; i < nodes.length; i++)
	      if (nodes[i].hasChildNodes())
	          ret.push(this.getInnerText(nodes[i]));
	      else if (nodes[i].tagName == "BR")
	          ret.push("\n");
	      else if (nodes[i].nodeType == Node.TEXT_NODE)
	          ret.push(nodes[i].nodeValue);
	      else if (nodes[i].alt)
	          ret.push(nodes[i].alt);
	   return ret.join('');
	},
	/** スレッドの情報をクリップボードにコピーします。*/
	copyThreadInfo: function() {
		this.setClipboard(this.title + "\n" + EXACT_URL + "\n");
	},
	/** 指定されたレス番号に対して返信します。
	 * @param {Number} number レス番号
	 */
	writeTo: function(number) {
		location.href = "bbs2ch:post:" + EXACT_URL + number;
	},
	/** すべての新着レスを既読状態にします。*/
	markAsRead: function() {
		var headers = ResNodes.getNewHeaders();
		for (var i = 0; i < headers.length; i++)
			headers.items(i).className = "resHeader";
	},
	/** 新着レスまでスクロールします。*/
	scrollToNewRes: function(){
		if(document.location.href.indexOf("#") != -1) return;
		if (this.option.match(/^\d+/)) { // we're in log pick-up mode
			window.scrollTo(0, 0);
			return;
		}
		if (Nodes.newMark) window.scrollTo(0, Nodes.newMark.offsetTop - (Nodes.header.offsetTop + Nodes.header.offsetHeight));
	},
	/** Favicon を設定します。
	 * @param {String}  filename アイコンのファイル名
	 * @param {Boolean} inactive 非アクティヴ状態かどうか
	 */
	setFavicon: function(filename, inactive) {
		var favicon = ResNodes.getFavicon();
		if (favicon) {
			var link = favicon.cloneNode(true);
			if (filename == "throbber") {
				link.href = SKIN_PATH + "img/favicon/favicon" + filename + ".gif";
			} else {
				if (AutoReload.enabled) {
					link.href = SKIN_PATH + "img/favicon/favicon" + filename + "auto" + (inactive ? "inactive" : "") + ".png";
				} else {
					link.href = SKIN_PATH + "img/favicon/favicon" + filename+ ".png";
				}
			}
			favicon.parentNode.appendChild(link);
			favicon.parentNode.removeChild(favicon);
		}
	},
	/** ステータスを設定して、表示を更新します。
	 * @param {String} updateStatus ステータス文字列
	 */
	setStatus: function(updateStatus) {
		/*
		ok				= (｀・ω・´)「OK」
		dat_down		= ( ｰωｰ)「DAT 落ち」
		not_modified 	= ( ｰωｰ)「新着なし」
		abone			= (´・ω・`)「あぼーん発生。右下のバッテンを押してログを消した後再読み込みしてね」
		error			= (´・ω・`)「エラー : %S」

		offline_mode	= ( ｰωｰ)「オフラインモード」
		log_pickup_mode = ( ｰωｰ)「ログピックアップモード」
		*/
		Nodes.resCount.textContent = "(" + this.countAll + ")";
		if (updateStatus) {
			//if (this.countUnread > 0) {
			if ((this.countUnread > 0) && (this.status != "( ｰωｰ)「ふー」")) {
				Nodes.statusText.className = "ok";
				this.status = this.countUnread + "件の新着レス";
				this.setFavicon("new");
			} else {
				switch(this.status){
				case "( ｰωｰ)「DAT 落ち」":
					Nodes.statusText.className = "warning";
					this.status = "DAT 落ち";
					this.setFavicon("warning");
					break;
				case "(｀・ω・´)「OK」": // for shitaraba
				case "( ｰωｰ)「新着なし」":
					Nodes.statusText.className = "ok";
					this.status = "新着なし";
					this.setFavicon("");
					break;
				case "(´・ω・`)「あぼーん発生。右下のバッテンを押してログを消した後再読み込みしてね」":
					Nodes.statusText.className = "warning";
					this.status = "あぼーんが発生しました。ログを消して、再読み込みして下さい。";
					this.setFavicon("warning");
					break;
				case "( ｰωｰ)「オフラインモード」":
					Nodes.statusText.className = "warning";
					this.status = "オフラインモード";
					this.setFavicon("warning");
					break;
				case "( ｰωｰ)「ログピックアップモード」":
					Nodes.statusText.className = "warning";
					this.status = "ログピックアップモード";
					this.setFavicon("warning");
					break;
				case "( ｰωｰ)「ふー」":
					Nodes.statusText.className = "warning";
					var now = new Date();
					this.status = "リロード制限（残り" + Math.ceil((5000 - (now.getTime() - this.date.getTime())) / 1000) + "秒）";
					this.setFavicon("warning");
					break;
				default:
					Nodes.statusText.className = "error";
					if (this.status.match("\(´・ω・`\)「エラー : (.+)」")) this.status = RegExp.$1;
					this.setFavicon("error");
				}
			}
		}
		//if (updateStatus) this.status = "+" + this.countUnread + "";
		Nodes.statusText.textContent = this.status;
		this.contractCaption();
	},
	/** スレッドタイトルのフォントにおける、指定された文字列の幅を取得します。
	 * @param  {String} str 文字列
	 * @return {Number} 幅
	 */
	getCaptionWidth: function(str) {
		var span  = document.createElement("span");
		span.visibility = "hidden";
		span.appendChild(document.createTextNode(str));
		Nodes.header.appendChild(span);
		var width = span.offsetWidth;
		Nodes.header.removeChild(span);
		return width;
	},
	/** スレッドタイトルを省略表示します。*/
	contractCaption: function() {
		var statusWidth = (Nodes.statusText.offsetLeft + Nodes.statusText.offsetWidth) - (Nodes.threadName.offsetWidth);
		var dotWidth    = this.getCaptionWidth("...");
		if (this.getCaptionWidth(ThreadDocument.title) + statusWidth > Nodes.command.offsetLeft) {
			for (var i = 1; i < this.title.length; i++) {
				if ((this.getCaptionWidth(this.title.substring(0,i)) + dotWidth + statusWidth) > Nodes.command.offsetLeft) {
					Nodes.threadName.textContent = this.title.substring(0,i-1) + "...";
					break;
				}
			}
		} else {
			Nodes.threadName.textContent = this.title;
		}
	},
	/** ID 欄を色分けし、発言回数を末端に追加します。*/
    colourIDs: function() {
    	ID.traverse();
    	//var idItems = ResNodes.getValidIDItems();
    	var idItems = ResNodes.getValidIDs();
   		var f = IDExtract.extract.bind(IDExtract);
		for (var i = 0; i < idItems.length; i++) {
			var node = idItems.items(i);
			var id = node.getAttribute("rel");
			var len = ID.items[id].length;
			if (len > 1) {
				node.textContent = "ID:" + id+ "(" + len + ")";
				node.className = (len > 2) ? "resID2" : "resID1";
				node.onclick = f;
			}
		}
    	//var idAnchorItems = ResNodes.getIDAnchorItems();
    	var idAnchorItems = ResNodes.getIDAnchors();
		for (var i = 0; i < idAnchorItems.length; i++) {
			var node = idAnchorItems.items(i);
			var pos = node.className.indexOf(" ");
			var className = (pos == -1) ? node.className : node.className.slice(0, pos);
			var id  = className.slice(6);
			if (ID.items[id]) {
				var len = ID.items[id].length;
				node.className = className + ((len > 2) ? " resID2" : " resID1");
				node.onclick = f;
			}
		}
	},
	/** 被参照レスに下線を引き、その下に件数を表示します。*/
	markTrackbackedResNumbers: function() {
		Trackback.traverse();
		var numbers = ResNodes.getUnmarkedNumbers();
		var div = document.createElement("div");
		div.className = "count";
		for (var i = 0; i < numbers.length; i++) {
			var node = numbers.items(i);
			var tb = Trackback.items[node.textContent];
			if (tb) {
				node.className = "trackback";
				var count = div.cloneNode(false);
				count.textContent = tb.length;
				node.parentNode.appendChild(count);
			}
		}
	},
	/** 設定に応じて、アンカーの属性を変更します。
	 * @param {element} contextNode 評価する親ノード
	 */
	modifyAnchors: function(contextNode) {
		var enableNewWindow = SkinPref.getBool("enableNewWindow", true);
		var enableNoReferer = SkinPref.getBool("enableNoReferer", true);
		var outLinkItems = ResNodes.getOutLinks(contextNode);
		for (var i = 0; i < outLinkItems.length; i++) {
			var node = outLinkItems.items(i);
			if (!node.rel) node.rel = node.href.replace(SERVER_URL, "");
			if (enableNewWindow) if (node.target != "_blank") node.target = "_blank";
			if ((node.offsetLeft + node.offsetWidth)  > document.body.clientWidth)
				node.innerHTML = (node.textContent.split("")).join("<wbr/>");
			if (node.href.indexOf("read.cgi") != -1)
				if (node.href.indexOf(SERVER_URL) == -1) {
					node.href = SERVER_URL + node.href;
					continue;
			}
			if (enableNoReferer) if (!ImagePopup.isImage(node.href)) {
				if (node.href.indexOf("ime.nu.html?url=") == -1) {
					if (node.textContent.indexOf("xn--") == -1) {
						node.href = SKIN_PATH + "ime.nu.html?url=" + node.href;
					} else {
						node.href = SKIN_PATH + "ime.nu.html?url=" + node.href + "&b64url=" + btoa(unescape(encodeURIComponent(node.href)));
					}
				}
			} 
		}
		
		var f = this.jumpTo.bind(this);
		var resAnchorItems = ResNodes.getResAnchors(contextNode);
		for (var i = 0; i < resAnchorItems.length; i++) {
			var node = resAnchorItems.items(i);
			node.href = "javascript:void(0)";
			node.onclick = f;
		}
	},
	/** XMLHttpRequest を利用して、動的にレスを挿入します。
	 * @param {Number}   start       挿入するレス番号の始点
	 * @param {Number}   end         挿入するレス番号の終点
	 * @param {Boolean}  before      レスを末端ではなく先頭に追加するかどうか
	 * @param {Boolean } scrollToTop 挿入後に先頭までスクロールさせるかどうか
	 * @param {Function} func        挿入後に実行する関数
	 */
	asyncInsert: function(start, end, before, scrollToTop, func) {
		var xmlHTTP = new XMLHttpRequest();
	    xmlHTTP.onreadystatechange = xmlEventHandler.bind(this);
	    function xmlEventHandler() {
	        if (xmlHTTP.readyState == 4) {
	            if (xmlHTTP.status == 200) {
					var divTemp  = document.createElement("div");
	                divTemp.innerHTML = xmlHTTP.responseText;
	                var content = ResNodes.getContent(divTemp);
	                content.id = "";
					var resItems = ResNodes.getSelectors();
					var resStart = ResNodes.getContainerByIndex(1);
        			resStart = (resStart) ? resItems.items(1) : resItems.items(0);
					if (resStart.parentNode.id.length == 0) resStart = resStart.parentNode;
					if (before) {
						Nodes.content.insertBefore(content, resStart);
					} else {
						Nodes.content.appendChild(content);
					}
					if (scrollToTop) {
						window.scrollTo(0,0);
					} else {
						var resStart = ResNodes.getContainerByIndex(start);
						if (resStart) window.scrollTo(0, resStart.offsetTop - (Nodes.header.offsetTop + Nodes.header.offsetHeight));
					}
					
					window.setTimeout((function() {
						this.modifyAnchors(content);
						this.colourIDs();
						this.markTrackbackedResNumbers();
					}).bind(this), 1);
					
					this.flagDigest = false;
					if (func) func();
					Nodes.statusText.className = "ok";
	    			this.setFavicon("");
				}
	        }
	    }
	    Nodes.statusText.className = "throbber";
	    this.setFavicon("throbber");
	    xmlHTTP.open("GET", SERVER_URL + EXACT_URL + start + "-" + end + "n" + "?lite=true", true);
	    xmlHTTP.overrideMimeType('text/xml');
	    xmlHTTP.send(null);
	},
	/** XMLHttpRequest を利用して、表示域外のレスをすべて表示します。*/
	showAll: function(){
		var nodes = ResNodes.getSelectors();
		for(var i = 0; i < nodes.length; i++) nodes.items(i).parentNode.style.display = "block";
		ThreadDocument.flagDigest = false;

		var resStart = ResNodes.getContainerByIndex(1);
		var addFirst = (resStart) ? false : true;
		resStart = (resStart) ? nodes.items(1) : nodes.items(0);
		var startIndex = ResNodes.getIndexBySelector(resStart);
		var endIndex   = ResNodes.getIndexBySelector(nodes.items(nodes.length - 1));
		//var endIndex   = Nodes.getIndexFromRes(resItems[resItems.length-1]);
        //if (!Nodes.getRes(1)) this.asyncInsert(1, 1, true, true);
        if (addFirst) this.asyncInsert(1, 1, true, true);
		if (startIndex > 2) this.asyncInsert(2, startIndex - 1, true, true);
		if (endIndex < this.countAll) this.asyncInsert(endIndex + 1, this.countAll, false, true);
	},
	/** XMLHttpRequest を利用して、更新をチェックし新着レスがあれば挿入します。
	 * @param {Boolean} noScroll 挿入後にスクロールしないかどうか
	 */
	reload: function(noScroll){
		if (this.countAll >= 1000) if (EXACT_URL.indexOf("2ch.net") != -1) return;
		var now = new Date();
		if ((now.getTime() - this.date.getTime()) < 5000) {
			this.status = "( ｰωｰ)「ふー」";
			this.setStatus(true);
			return;
		}
		Nodes.statusText.className = "throbber";
	    this.setFavicon("throbber");
		this.date = now;
		this.markAsRead();
		var xmlHTTP = new XMLHttpRequest();
	    xmlHTTP.onreadystatechange = xmlEventHandler.bind(this);
	    function xmlEventHandler() {
	        if (xmlHTTP.readyState == 4) {
	            if (xmlHTTP.status == 200) {
					var divTemp  = document.createElement("div");
	                divTemp.innerHTML = xmlHTTP.responseText;
	                
	                var content = ResNodes.getContent(divTemp);
	                content.id = "";
	                var resItems = ResNodes.getContainers(divTemp);
	                // 更新がなければ最後のレスだけ帰ってくる
	                if (resItems.length <= 1) {
	                	this.status = "( ｰωｰ)「新着なし」";
	                	this.countUnread = 0;
	                	this.countRead   = this.countAll;
						this.setStatus(true);
	                	return;
	                }
	                // 一個余分についてきてしまうので、消す
	                var firstItem = ResNodes.getSelectors(content).first;
	                if (firstItem) content.removeChild(firstItem);
	                
                	this.countRead   = this.countAll;
                	//this.countUnread = resItems.length ;
                	this.countUnread = resItems.length - 1;
                	this.countAll    = this.countRead + this.countUnread;
					this.setStatus(true);
					Nodes.content.appendChild(content);
					
					window.setTimeout((function() {
						this.modifyAnchors(content);
						this.colourIDs();
						this.markTrackbackedResNumbers();
					}).bind(this), 1);
					
					if (!noScroll) {
						var resStart = ResNodes.getSelectors(content).first;
						if (resStart) window.scrollTo(0, resStart.offsetTop - (Nodes.header.offsetTop + Nodes.header.offsetHeight));
					}
					this.flagDigest = false;
				}
	        }
	    }
	    xmlHTTP.open("GET", SERVER_URL + EXACT_URL + (ThreadDocument.countAll + 1) + "-" + "?lite=true", true);
	    xmlHTTP.send(null);
	},
	/** 全角文字で表記された数字を、数値型に変換します。
	 * @param  {String} str 全角数字が含まれた文字列
	 * @return {Number} 変換された数値
	 */
	toNarrow: function(str) {
		return parseInt(str.replace(/([０-９])/g, function(n){ return String.fromCharCode(n.charCodeAt(0) - 0xFEE0); }));
	},
	/** 指定されたレスまでスクロールします。アンカーの click イベントから呼ばれます。
	 * @param {event} e イベント
	 */
	jumpTo: function(e){
		var node = e.target;
		var expPointerAnchor  = new RegExp(/(\d{1,4})-?(\d{0,4})/);
		var expNameAnchor     = new RegExp(/^(\d{1,4})-?(\d{0,4})$/);
		switch (node.className) {
		case "resPointer":
			if (node.textContent.match(expPointerAnchor)) {
				var start = this.toNarrow(RegExp.$1);
				var end   = RegExp.$2 ? this.toNarrow(RegExp.$2) : start;
			}
			break;
		case "resName":
			if (node.textContent.match(expNameAnchor)) {
				var start = this.toNarrow(RegExp.$1);
				var end   = RegExp.$2 ? this.toNarrow(RegExp.$2) : start;
			}
			break;
		default:
			return;
		}
		var resTarget = Nodes.getRes(start);
		if (resTarget) {
			var newStart = 0;
			for(var i = start + 1; i <= end; i++) {
				if (!this.getRes(i)) {
					newStart = i;
					break;
				}
			}
			if (newStart == 0) {
				window.scrollTo(0, resTarget.offsetTop - (Nodes.header.offsetTop + Nodes.header.offsetHeight));
				return;
			}
			start = newStart;
		}
		var resItems = Nodes.getResItems();
		var resStart = Nodes.getRes(1);
        resStart = (resStart) ? resItems[1] : resItems[0];
		var startIndex = Nodes.getIndexFromRes(resStart);
		var endIndex   = Nodes.getIndexFromRes(resItems[resItems.length-1]);
		if (start < startIndex) this.asyncInsert(start, startIndex - 1, true, false);
		if (endIndex < end)     this.asyncInsert(endIndex, end, true, false);
	}
};

/**
 * ID を走査して、レス番号と ID の対応テーブルを管理します。
 * @static
 */
var ID = {
	/** レス番号と ID の対応テーブル
	 * @type Object
	 * @example
	 * {
	 *     "hogehoge": [1, 3, 5],
	 *     "foobar":   [200, 201, 208]
	 * }
	 */
	items: [],
	/** 有効な ID の総個数
	 * @type Number
	 */
	count: null,
	/** テーブルに項目を追加します。
	 * @param {String} id ID
	 * @param {Number} index レス番号
	 */
	add: function(id, index) {
		if (!this.items[id]) this.items[id] = [];
		this.items[id].push(index);
	},
	/** ID を走査して、対応テーブルを作ります。*/
	traverse: function() {
		//var resItems = ResNodes.getResItems();
		//var resItems = ResNodes.getContainers();
		var idItems = ResNodes.getValidIDs();
		//if (resItems.length == this.count) return;
		if (idItems.length == this.count) return;
		this.count = idItems.length;
		this.items = [];
		for (var i = 0; i < idItems.length; i++) {
			this.add(idItems.items(i).getAttribute("rel"), ResNodes.getIndexByID(idItems.items(i)));
		}
	}
}

/**
 * レスアンカーを走査して、レス番号とレス元の番号との対応テーブルを管理します。
 * @static
 */
var Trackback = {
	/** レス番号とレス元の番号との対応テーブル
	 * @type Object
	 * @example
	 * {
	 *     "1": [2, 3, 4, 5],
	 *     "4": [5],
	 *     "6": [8]
	 * }
	 */
	items: [],
	/** 走査したレスの件数
	 * @type Number
	 */
	count: null,
	/** 全角数字を半角数字に変換するテーブル
	 * @type Number
	 */
	narrowTable: {"１":"1","２":"2","３":"3","４":"4","５":"5","６":"6","７":"7","８":"8","９":"9","０":"0"},
	/** 全角文字で表記された数字を、数値型に変換します。
	 * @param  {String} str 全角数字が含まれた文字列
	 * @return {Number} 変換された数値
	 */
	toNarrow: function(str) {
		//return parseInt(str.replace(/([０-９])/g, function(n){ return String.fromCharCode(n.charCodeAt(0) - 0xFEE0); }));
		var array = str.split("");
		for (var i = 0; i < array.length; i++) str[i] = this.narrowTable[str];
		return parseInt(array.join(""));
	},
	/** テーブルに項目を追加します。
	 * @param {Number} destIndex レス先のレス番号
	 * @param {Number} srcIndex  レス元のレス番号
	 */
	add: function(destIndex, srcIndex) {
		if (this.items[destIndex]) {
			for (var i = 0; i < this.items[destIndex].length; i++)	if (this.items[destIndex][i] == srcIndex) return;
		} else {
			this.items[destIndex] = [];
		}
		this.items[destIndex].push(srcIndex);
	},
	/** 指定のレス番号についたレスの数を取得します。
	 * @param  {Number} index レス番号
	 * @return {Number} レスの数
	 */
	getCount: function(index) {
		if (!this.items[index]) return 0;
		return this.items[index].length;
	},
	/** 指定のレス番号についたレスについて、レスアンカーのリストを生成して返します。
	 * @param  {Number} index レス番号
	 * @return {DocumentFragment} 要素
	 */
	getAnchorElements: function(index) {
		if (!this.items[index]) return;
		var content = document.createDocumentFragment();
		var a = document.createElement("a");
		a.className = "resPointer";
		a.href    = "javascript:void(0)";
		a.onclick = (function(e) { ThreadDocument.jumpTo(e) }).bind(this);
		for (var i = 0; i < this.items[index].length; i++){
			var item = a.cloneNode(false);
			item.appendChild(document.createTextNode(">>" + this.items[index][i]));
			content.appendChild(item);
		}
		return content;
	},
	/** 指定の要素に、逆参照情報を追加します。
	 * @param  {element} content 要素
	 * @param  {Number}  index   レス番号
	 */
	appendTo: function(content, index) {
		if (!this.items[index]) return;
		var span = document.createElement("span");
		span.className = "trackBack";
		span.appendChild(document.createTextNode("(参照:"));
		span.appendChild(this.getAnchorElements(index));
		span.appendChild(document.createTextNode(")"));
		content.appendChild(span);
	},
	// XPath で分岐を減らして多少高速化しますた(2007/08/30)
	/** レスを走査して、対応テーブルを作ります。*/
	traverse: function() {
		const TRACKBACK_LIMIT = 20;
		//var r = Nodes.getResItems();
		var r = ResNodes.getContainers();
		if (r.length == this.count) return;
		this.count = r.length;
		var expAnchor = new RegExp(/(\d{1,4})-?(\d{0,4})/);
		this.items = new Array();
		//var anchors = ResNodes.getResAnchorItems();
		var anchors = ResNodes.getResAnchors();
		for (var i = 0; i < anchors.length; i++) {
			if (anchors.items(i).textContent.match(expAnchor)) {
				var start = this.toNarrow(RegExp.$1);
				var end   = RegExp.$2 ? this.toNarrow(RegExp.$2) : start;
				if ((end - start) < TRACKBACK_LIMIT) {
					for(var destIndex = start; destIndex <= end; destIndex++) {
						var srcIndex = anchors.items(i).parentNode.id.slice(4); // body
						if (srcIndex) this.add(destIndex, srcIndex);
					}
				}
			}
		}
	}
}

/**
 * ID の抽出機能を管理します。
 * @static
 */
var IDExtract = {
	/** 抽出中の ID (obsolete)
	 * @type String
	 */
	id: "",
	/** スレッド表示のオプションで抽出が要求された場合、指定された ID を持つレスのみを表示します。(obsolete)*/
	initialise: function() {
		if (!location.href.match(/\?id=/)) return;
		id = RegExp.rightContext;
		if (id.length < 4) return;
		if (id.match(/^ID:\?\?\?/)) return;
		var resItems = Nodes.getResItems();
		var	idItems  = Nodes.getIDItems();
		var idCount  = 0;
		Trackback.traverse();
		for (var i = 0; i < resItems.length; i++) {
			if (idItems[i].textContent.indexOf(id,0) != -1){
				Trackback.appendTo(resItems[i], Nodes.getIndexFromRes(resItems[i]));
				resItems[i].style.display = "block";
				idCount++;
			} else {
				resItems[i].style.display = "none";
			}
		}
		ThreadDocument.status = id + " のレス (" + idCount + "回)";
		ThreadDocument.setStatus();
		IDExtract.id = id;
	},
	/** 指定された ID を抽出します。アンカーの click イベントから呼ばれます。
	 * @param {event} e イベント
	 */
	extract: function(e) {
    	if (SkinPref.getBool("enableIDPopupOnClick", false) && !e.ctrlKey) return;
		if (!FindBox.isVisible) FindBox.show();
		Nodes.findBoxText.value = "id:"  + e.target.getAttribute("rel");
		FindBox.find();
	}
};

/**
 * ショートカットキーを管理します。
 * @static
 */
var KeyInput = {
	/** イベントリスナを登録します。*/
	initialise: function() {
		window.addEventListener("keydown", this.onKeyDown.bind(this), false);
	},
	/** keydown イベントを処理します。
	 * @param {event} e イベント
	 */
	onKeyDown: function(e) {
		this.reloadProc(e);
		this.showAllProc(e);
		this.writeProc(e);
		if (e.keyCode == 70) if (e.altKey && e.ctrlKey) FindBox.show();
	},
	/** [書き込みウィザード] を開く処理を定義します。
	 * @param {event} e イベント
	 */
	writeProc: function(e){
		if (e.ctrlKey) if (e.keyCode == 13) {
			location.href = "bbs2ch:post:" + EXACT_URL;
			e.preventDefault();
		}
	},
	/** [すべて表示] を実行する処理を定義します。
	 * @param {event} e イベント
	 */
	showAllProc: function(e){
		if (e.ctrlKey) if (e.keyCode == 229 || e.keyCode == 32) {
			ThreadDocument.showAll();
			e.preventDefault();
		}
	},
	/** 更新を実行する処理を定義します。
	 * @param {event} e イベント
	 */
	reloadProc: function(e){
		if (!SkinPref.getBool("enableHookReload", true)) return;
		if (e.keyCode == 116) {
			ThreadDocument.reload();
			e.preventDefault();
		}
	}
};

/**
 * [検索/抽出] 機能を管理します。
 * @static
 * @property {Boolean} isVisible 検索/抽出ボックスが表示されているかどうか
 */
var FindBox = {
	/** 既に初期化済みかどうか
	 * @type Boolean
	 */
	initialised: false,
	/** 検索/抽出実行前のスクロール位置
	 * @type Number
	 */
	scrollY: -1,
	get isVisible() {
		return (Nodes.findBox.style.display == "block");
	},
	/** keydown イベントを処理します。
	 * @param {event} e イベント
	 */
	onKeyPress: function(e) {
		switch (e.keyCode) {
		case 13:
			if (e.target == Nodes.findBoxText) this.find();
			break;
		case 27: //ESC
			if (this.isVisible) this.clear();
			break;
		}
	},
	/** click イベントを処理します。
	 * @param {event} e イベント
	 */
	onClick:     function(e) {
		if (e.target.id == "findboxClear") this.clear();
	},
	/** mouesdown イベントを処理します。
	 * @param {event} e イベント
	 */
	onMouseDown: function(e) {
		if (!e.target.id.match(/(find|findbox|findboxClear|findboxText)/)) 
			if (!Nodes.findBoxText.value.length) this.hide();
	},
	/** 検索/抽出ボックスを表示します。
	 * @param {event} e イベント
	 */
	show: function() {
		if (Nodes.findBox.style.display != "block") {
			Nodes.findBox.style.display = "block";
			Nodes.findBoxText.focus();
		} else {
			this.hide();
		}
		if (!this.initialised) {
			this.initialised = true;
			window.addEventListener("click",     this.onClick.bind(this), false);
			window.addEventListener("mousedown", this.onMouseDown.bind(this), false);
			window.addEventListener("keypress",  this.onKeyPress.bind(this), false);
		}
	},
	/** 検索/抽出ボックスを非表示します。
	 * @param {event} e イベント
	 */
	hide: function() {
		Nodes.findBox.style.display = "none";
	},
	/** 検索/抽出ボックスをクリアします。
	 * @param {event} e イベント
	 */
	clear: function() {
		Nodes.findBoxText.value = "";
		Nodes.findBoxText.className = "";
		if (SkinPref.getBool("enableFindHighlight", false)) window.setTimeout(this.unhighlight, 1);
		var items = Nodes.getResItems();
		for (var i = 0; i < items.length; i++) {
			items[i].style.display = "block";
		}
		if (this.scrollY != -1) window.scrollTo(0, this.scrollY);
		this.scrollY = -1;
	},
	/** 正規表現に使用される文字列をエスケープします。
	 * @param  {String} str 文字列
	 * @return {String} エスケープされた文字列
	 */
	escapeExpression: function(str) {
		return str.replace(/([.*+?^${}()|[\]\/\\])/g, '\\$1');
	},
	/** 指定された文字列を強調表示します。
	 * @param  {String} str 文字列
	 */
	highlight: function(str) {
		console.time("highlight");
		var x = window.scrollX;
		var y = window.scrollY;
		var strong = document.createElement("strong");
		while (window.find(str)) {
			var range = window.getSelection().getRangeAt(0);
			window.setTimeout(function(r){ r.surroundContents(strong.cloneNode(false)); }, 1, range);
		}
		window.getSelection().removeAllRanges();
		window.scrollTo(x, y);
		console.timeEnd("highlight");
	},
	/** 指定された文字列の強調表示を解除します。
	 * @param  {String} str 文字列
	 */
	unhighlight: function(str) {
		console.time("unhighlight");
		var result = document.evaluate('//strong', document, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
		for (var i = 0; i < result.snapshotLength; i++) {
			var node = result.snapshotItem(i);
			window.setTimeout(function(n){
				var doc = document.createDocumentFragment();
				for (var i = 0; i < n.childNodes.length; i++) { 
					doc.appendChild(n.childNodes[i]);
				}
				n.parentNode.replaceChild(doc, n);
			}, 1, node);
		}
		console.timeEnd("unhighlight");
	},
	/** 検索/抽出を実行します。*/
	find: function() {
		var scrollY = window.scrollY;
		var items = Nodes.getResItems();
		var isFound = false;

		var xpathResItems = ResNodes.getContainers();
		var xpaths = { "name": ResNodes.getNames,
					   "mail": ResNodes.getMails,
					   "date": ResNodes.getDates,
					   "id":   ResNodes.getIDs,
					   "beid": ResNodes.getBeIDs,
					   "body": ResNodes.getBodies};
		if (Nodes.findBoxText.value.match(/^(name|mail|date|id|body|related):(.*)/i)) {
			var verb = RegExp.$1.toLowerCase();
			switch (verb) {
				case "related":
					break;
				case "id":
					var xpath = xpaths["id"]();
					for (var i = 0; i < xpath.length; i++) {
						var id = xpath.items(i).getAttribute("rel");
						if ((id.length >= 8)  && (id.indexOf(RegExp.$2) != -1)) {
							isFound = true;
							xpathResItems.items(i).style.display = "block";
						} else {
							xpathResItems.items(i).style.display = "none";
						}
					}
					break;
				default:
					var xpath = xpaths[verb]();
					for (var i = 0; i < xpath.length; i++)
					{
						if ((xpath.items(i).textContent).indexOf(RegExp.$2) != -1) {
							isFound = true;
							xpathResItems.items(i).style.display = "block";
						} else {
							xpathResItems.items(i).style.display = "none";
						}
					}
					break;
			}
		} else {
			for (var i = 0; i < xpathResItems.length; i++) {
				var re = xpathResItems.items(i).textContent.match(this.escapeExpression(Nodes.findBoxText.value), "i");
				if (re) {
					isFound = true;
					xpathResItems.items(i).style.display = "block";
				} else {
					xpathResItems.items(i).style.display = "none";
				}
			}
		}
		Nodes.findBoxText.className = (isFound) ? "found" : "notfound";
		if (isFound) {
			if (this.scrollY == -1) this.scrollY = scrollY;
			window.scrollTo(0, 0);
			if (SkinPref.getBool("enableFindHighlight", false)) window.setTimeout(this.highlight, 1, Nodes.findBoxText.value);
		}
	}
};

/**
 * [自動更新] 機能を管理します。
 * @static
 * @property {Boolean} enabled 自動更新が有効かどうか
 */
var AutoReload = {
	/** 自動更新に使うタイマー ID
	 * @type Number
	 */
	timerID: 0,
	/** 開いているスレッドがアクティヴ状態かどうか
	 * @type Boolean
	 */
	isActive: true,
	/** 実際に更新を要求するかどうか
	 * @type Boolean
	 */
	requestReload: false,
	/** このスレッドが閉じられようとしているかどうか
	 * @type Boolean
	 */
	isUnloading: false,
	/** インターバルの配列
	 * @type Array
	 */
	INTERVAL: [15000, 30000, 60000],
	/** 各種イベントリスナを登録し、初期化を行います。*/
	initialise: function() {
		if (SkinPref.getBool("enableAutoReloadOnLiveThread", false)) 
			if (EXACT_URL.indexOf("http://live") != -1) this.enabled = true;
		if (!SkinPref.getBool("enableAutoReloadWhenInactive", false)) {
			window.addEventListener("beforeunload", (function() { this.isUnloading = true }).bind(this), false);
			window.addEventListener("focus", (function() {
					if (this.enabled && !this.isActive) {
						this.isActive = true;
						ThreadDocument.setFavicon("", false);
						if (this.requestReload) {
							if (!this.isUnloading) {
								this.timerProc();
								this.enabled = true;
							}
							this.requestReload = false;
						}
					}
				}).bind(this), false);
			window.addEventListener("blur", (function() {
					if (this.enabled && this.isActive) {
						this.isActive = false;
						ThreadDocument.setFavicon("", true);
					}
				}).bind(this), false);
		}
	},
	/** タイマーのプロシージャです。*/
	timerProc: function() {
		if (this.isActive) {
			var items = Nodes.getResItems();
			var lastItem = items[items.length - 1];
			ThreadDocument.reload(!(window.scrollY + window.innerHeight > lastItem.offsetTop + lastItem.offsetHeight));
			this.requestReload = false;
		} else {
			this.requestReload = true;
		}
	},
	get enabled() {
		return (this.timerID != 0);
	},
	set enabled(value) {
		if (this.timerID) {
			window.clearInterval(this.timerID);
			this.timerID = 0;
		}
		if (value) {
			this.timerID = window.setInterval(this.timerProc.bind(this), this.INTERVAL[SkinPref.getInt("valueAutoReloadInterval", 1)]);
		}
		ThreadDocument.setFavicon("");
	}
};

/**
 * [要約] 機能を管理します。
 * @static
 * @property {Boolean} enabled 要約が有効かどうか
 */
var Digest = {
	_enabled: false,
	get enabled() {
		return this._enabled;
	},
	set enabled(value) {
		this._enabled = value;
		var expSingleAnchor = new RegExp(/(>?>|＞)(\d{1,4})/);
		Trackback.traverse();
		if (!Trackback.items) return;
		//var bodyItems = ResNodes.getBodyItems();
		var bodyItems = ResNodes.getBodies();
		if (this._enabled) {
			for (var i = 0; i < bodyItems.length; i++) {
				var body = bodyItems.items(i);
				body.parentNode.style.display = ((Trackback.items[body.id.slice(4)]) || (body.textContent.match(expSingleAnchor))) ? "block" : "none"
			}
		} else {
			for (var i = 0; i < bodyItems.length; i++) {
				bodyItems.items(i).parentNode.style.display = "block";
			}
		}
	}
};

/**
 * しおり機能を管理します。
 * @static
 */
var Bookmark = {
	/** しおりが挿んであるレス番号
	 * @return Number
	 */
	index: 0,
	/** しおり要素の幅
	 * @return Number
	 */
	width: 0,
	/** イベントリスナを登録し、しおりが挿んであるレスが非表示であれば表示させます。
	 * @return {Number} しおりが挿んであるレス番号
	 */
	initialise: function() {
		window.addEventListener("mouseover", this.onMouseOver.bind(this), false);
		window.addEventListener("mouseout",  this.onMouseOut.bind(this), false);
		window.addEventListener("click",     this.onClick.bind(this), false);
		if (ThreadDocument.boardName && ThreadDocument.threadID) {
			this.index = SkinPref.getInt("valueBookmarkIndex@" + ThreadDocument.boardName + "/" + ThreadDocument.threadID, 0);
			if (this.index) {
				var resStart = Nodes.getRes(this.index);
				if (resStart) {
					window.scrollTo(0, resStart.offsetTop - (Nodes.header.offsetTop + Nodes.header.offsetHeight));
					this.insert(this.index, true);
				} else {
					var resItems = Nodes.getResItems();
					var resStart = Nodes.getRes(1);
        			resStart = (resStart) ? resItems[1] : resItems[0];
					ThreadDocument.asyncInsert(this.index, Nodes.getIndexFromRes(resStart), true, false,
						(function(){this.insert(this.index, true)}).bind(this) );
				}
			}
		}
		return this.index;
	},
	/** mouseover イベントを処理します。
	 * @param {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		if (node.className == "resContainer") {
			if (e.clientX > document.body.clientWidth - 64) {
				node.style.cursor = "pointer";
				node.title = (Nodes.getIndexFromRes(node) == this.index) ? "しおりを外す" : "しおりを挿入";
			} else {
				if (node.style.cursor == "pointer") {
					node.style.cursor = "auto";
					node.title = "";
				}
			}
		}
	},
	/** mouseout イベントを処理します。
	 * @param {event} e イベント
	 */
	onMouseOut: function(e) {
		var node = e.target;
		if (node.className == "resContainer") {
			if (node.style.cursor == "pointer") {
				node.style.cursor = "auto";
				node.title = "";
			}
		}
	},
	/** click イベントを処理します。
	 * @param {event} e イベント
	 */
	onClick: function(e) {
		var node = e.target;
		if (node.className == "resContainer") if (e.clientX > document.body.clientWidth - 64) {
			var index = Nodes.getIndexFromRes(node);
			if (index == this.index) {
				this.hide();
			} else {
				this.insert(index);
			}
		}
	},
	/** しおりを挿入します。
	 * @param {Number}  index       挿入するレス番号
	 * @param {Boolean} noAnimatoin アニメーションを無効にするかどうか
	 */
	insert: function(index, noAnimation) {
		var node = Nodes.getRes(index);
		if (node) {
			var div = document.getElementById("bookmark");
			if (div) div.parentNode.removeChild(div);
			var div = document.createElement("div");
			div.id = "bookmark";
			div.title = "しおりを外す";
			div.onmousedown = (function(){ this.hide() }).bind(this);
			this.index = index;
			SkinPref.setInt("valueBookmarkIndex@" + ThreadDocument.boardName + "/" + ThreadDocument.threadID, this.index);
			node.parentNode.insertBefore(div, node);
			this.width = div.clientWidth;
			if (!noAnimation) this.show();
		}
	},
	/** しおりを実際に表示します。
	 * @param {Number} step しおりの加速度
	 */
	show: function(step) {
		if (!step) step = 1;
		var div = document.getElementById("bookmark");
		if (div) {
			if (step < this.width) {
				div.style.width = step + "px";
				window.setTimeout(this.show.bind(this), 1, step * 4);
			} else {
				div.style.width = this.width + "px";
			}
		}
	},
	/** しおりを非表示します。
	 * @param {Number} step しおりの加速度
	 */
	hide: function(step) {
		if (!step) step = 1;
		var div = document.getElementById("bookmark");
		if (div) {
			if (step < this.width) {
				div.style.width = (this.width - step) + "px";
				window.setTimeout(this.hide.bind(this), 1, step * 4);
			} else {
				this.index = 0;
				SkinPref.remove("valueBookmarkIndex@" + ThreadDocument.boardName + "/" + ThreadDocument.threadID);
				div.parentNode.removeChild(div);
			}
		}
	}
};


/**
 * 左右カーソルキーによるスムーズスクロール機能を管理します。
 * @static
 */
 // スクロール処理中に更にキーが押されたときでも yield でそれっぽく処理できるようになった
var PageScroller = {
	/** スクロール位置
	 * @type Number
	 */
	y: null,
	/** 現在のスクロール対象のレス番号
	 * @type Number
	 */
	index: null,
	/** 速度
	 * @type Number
	 */
	currentVelocity: 0,
	/** 初速
	 * @type Number
	 */
	initialVelocity: 0,
	/** ジェネレータ
	 * @type Generator
	 */
	generator: null,
	/** まだスクロール中かどうか
	 * @type Boolean
	 */
	isBusy: false,
	/** ジェネレータの実行をリスタートすべきかどうか
	 * @type Boolean
	 */
	shouldRestart: false,
	/** イベントリスナを登録します。*/
	initialise: function() {
		window.addEventListener("keydown", this.onKeyDown.bind(this), false);
	},
	/** keydown イベントを処理します。
	 * @param {event} e イベント
	 */
	onKeyDown: function(e) {
		var node = e.target;
		if (node.tagName == 'TEXTAREA' || node.tagName == 'INPUT' || e.shiftKey || e.ctrlKey || e.altKey) return;
		switch (e.which) {
		case 37:
			e.preventDefault();
			this.prev();
			break;
		case 39:
			e.preventDefault();
			this.next();
		}
	},
	/** 前のレスへスクロールします。*/
	prev: function() {
		if (this.isBusy) {
			//if (this.index > 0) this.index--;
			
			this.index--;
			var resItems = Nodes.getResItems();
			for (var i = this.index ; i >= 0 ; i--) {
				if (resItems[i].style.display != "none") break;
				this.index--;
			}
			/*
			var resItems = Nodes.getResItems();
			for (var i = this.index - 1; i >= 0; i--) {
				if (resItems[i].style.display == "block") {
					this.index = i;
					break;
				}
			}
			*/
			var resItem = resItems[this.index];
			//var resItem = Nodes.getResItems()[this.index];
			if (resItem) {
				this.y = resItem.offsetTop - (Nodes.header.offsetTop + Nodes.header.offsetHeight);
				this.requestScroll();
			}
		} else {
			var top    = window.pageYOffset + (Nodes.header.offsetTop + Nodes.header.offsetHeight);
			var bottom = window.pageYOffset + window.innerHeight;
			var resItems = Nodes.getResItems();
			for (var i = resItems.length - 1; i >= 0 ; --i) {
				if ((resItems[i].style.display != "none") && (resItems[i].offsetTop < top)) {
					var resTop = resItems[i];
					if (resTop) {
						this.index = i;
						//this.lastDirection = "prev";
						this.y = resTop.offsetTop - (Nodes.header.offsetTop + Nodes.header.offsetHeight);
						this.requestScroll();
					}
					break;
				}
			}
		}
	},
	/** 次のレスへスクロールします。*/
	next: function() {
		//if ((this.lastDirection == "next") && (this.isBusy)) {
		if (this.isBusy) { 
			this.index++;
			var resItems = Nodes.getResItems();
			for (var i = this.index ; i < resItems.length; i++) {
				if (resItems[i].style.display != "none") break;
				this.index++;
			}
			var resItem = resItems[this.index];
			if (resItem) {
				//this.y = ((resItem.offsetTop + resItem.offsetHeight) - window.innerHeight);
				this.y = resItem.offsetTop - (Nodes.header.offsetTop + Nodes.header.offsetHeight);
				this.requestScroll();
			}
			
		} else {
			var top    = window.pageYOffset + (Nodes.header.offsetTop + Nodes.header.offsetHeight);
			var bottom = window.pageYOffset + window.innerHeight;
			var resItems = Nodes.getResItems();
			/*
			for (var i = 0; i < resItems.length; ++i) {
				if (resItems[i].offsetTop > bottom) {
					var resTop = resItems[i-1];
					if (resTop) {
						this.index = i - 1;
						this.lastDirection = "next";
						this.y = ((resTop.offsetTop + resTop.offsetHeight) - window.innerHeight);
						this.requestScroll();
					}
					break;
				}
			}
			*/
			for (var i = 0; i < resItems.length ; i++) {
				if ((resItems[i].style.display != "none") && (resItems[i].offsetTop > top)) {
					var resTop = resItems[i];					
					if (resTop) {
						this.index = i;
						//this.lastDirection = "next";
						this.y = resTop.offsetTop - (Nodes.header.offsetTop + Nodes.header.offsetHeight);
						this.requestScroll();
					}
					break;
				}
			}
			if (!resTop) {
				this.index = Nodes.getResItems().length - 1;
				this.y = document.body.offsetHeight;
				this.requestScroll();
			}
			
		}
	},
	/** スクロールを要求します。*/
	requestScroll: function() {
		if (!(SkinPref.getBool("enableSmoothScroll", true))) return window.scrollTo(0, y);
		if (this.isBusy) {
			this.shouldRestart = true;
		} else {
			this.scroll();
		}
	},
	/** スクロール処理を実行・管理します。*/
	scroll: function() {
		this.generator = this.scrollProc();
		this.isBusy = true;
		var y = this.y;
		var id = setInterval((function() {
			try {
				if (this.shouldRestart) {
					this.generator.close();
					this.generator = this.scrollProc();
					this.shouldRestart = false;
				}
				this.generator.next();
				this.generator.next();
				this.generator.next();
				this.generator.next();
			} catch(e) {
				clearInterval(id);
				this.isBusy = false;
			}
		}).bind(this) , 0, this.y);
	},
	/** 実際のスクロール処理を行います。*/
	scrollProc: function() {
		var y1 = window.pageYOffset;
		var y2 = parseInt(this.y);
		if (y2 < 0) y2 = 0;
		if ((y2 + window.innerHeight) > document.body.offsetHeight) y2 = document.body.offsetHeight - window.innerHeight;
		var delta = y2 - y1;
		switch (SkinPref.getInt("valueSmoothScrollFrames", 1)) {
		case 0:
			var steps = 16;  break;
		case 1:
			var steps = 24;  break;
		case 2:
			var steps = 32; break;
		default:
			var steps = 24;
		}
		//var steps = 32;
		this.initialVelocity = this.currentVelocity;

		var a1 = (((delta / 2) - this.initialVelocity) * 2) / Math.pow(steps / 2, 2);
		var ha1 = a1 / 2;

		for (var x = 0; x < steps / 2; ++x) {
			yield; 
			//window.scrollTo(0, (this.initialVelocity * x) + y1 + (ha1 * Math.pow(x, 2)));
			window.scrollTo(0, this.initialVelocity + y1 + (ha1 * Math.pow(x, 2)));
			this.currentVelocity = (a1 * x); //+ this.initialVelocity;
		}
		var a2 = (((delta / 2) - 0) * 2) / Math.pow(steps / 2, 2);
		var ha2 = a2 / 2;
		for (var x = steps / 2 - 1; x >= 0; --x) {
			yield;
			window.scrollTo(0, y2 - (ha2 * Math.pow(x, 2)));
			this.currentVelocity = (a2 * x);
		}					
		window.scrollTo(0, y2);
		this.initialVelocity = 0;
		this.currentVelocity = 0;
	},
}

/**
 * 複数レスの選択を管理します。
 * @static
 * @property {Number} length 選択されたレスの数
 */
var MultipleResSelector = {
	/** マウスボタンが押されているかどうか
	 * @type Boolean
	 */
	isButtonDown: false,
	/** 選択が開始された要素
	 * @type element
	 */
	nodeFrom: null,
	/** 選択モード (true: 選択, false: 非選択)
	 * @type Boolean
	 */
	modeSelect: false,
	get length() {
		var items = ResNodes.getSelectors().selected;
		return items.length;
	},
	/** 選択されたレスのリストを、カンマ区切りの文字列を取得します。
	 * @return {String} 文字列
	 */
	getAnchorString: function() {
		var items = ResNodes.getSelectors().selected;
		if (!items.length) return;
		var indexes = [];
		var node = items.items(0);
		var prevIndex = ResNodes.getIndexByContainer(node.firstChild);
		var seqStart = 0;
		for (var i = 1; i < items.length; i++) {
			var node = items.items(i);
			var index = ResNodes.getIndexByContainer(node.firstChild);
			if (index == (prevIndex + 1)) {
				seqStart = (seqStart > 0) ? seqStart : prevIndex;
			} else {
				if (seqStart > 0) {
					indexes.push(seqStart + "-" + prevIndex);
					seqStart = 0;
				} else {
					indexes.push(prevIndex);
				}
			}
			prevIndex = index;
		}
		indexes.push((seqStart > 0) ? seqStart + "-" + index : index);
		return indexes.join(",");
	},
	/** 要素の表示を強制します。
	 * @param {element} node 要素
	 */
	ensureVisible: function(node) {
		if (node.offsetTop < (window.scrollY + Nodes.header.offsetTop + Nodes.header.offsetHeight)) {
			window.scrollTo(0, node.offsetTop - (Nodes.header.offsetTop + Nodes.header.offsetHeight));
		} else if ((node.offsetTop + node.offsetHeight) > (window.scrollY + window.innerHeight)) {
			window.scrollTo(0, node.offsetTop - window.innerHeight + node.offsetHeight);
		}
	},
	/** レスの選択状態を解除します。*/
	clear: function() {
		var items = ResNodes.getSelectors().selected;
		for (var i = 0; i < items.length; i++) {
			var node = items.items(i);
			node.className = "resUnselected";
		}
	},
	/** イベントリスナを登録します。*/
	initialise: function() {
		Nodes.content.addEventListener("mousedown",   this.onMouseDown.bind(this), true);
		Nodes.content.addEventListener("mouseup",     this.onMouseUp.bind(this), true);
		Nodes.content.addEventListener("mouseover",   this.onMouseOver.bind(this), true);
		Nodes.content.addEventListener("contextmenu", this.onContextMenu.bind(this), true);
	},
	/** イベントから、マウスの座標が有効範囲内にあるかどうかを取得します。
	 * @return {Boolean} 有効範囲内であれば true
	 */
	isEventInRange: function(e) {
		return ((e.target.className == "resContainer") && (e.clientX <= 48)) ? true : false;
	},
	/** レス要素を選択します。
	 * @param {element} node 要素
	 */
	selectNodes: function(nodeTo) {
		this.clear();
		//var items = ResNodes.getResItemSelectors();
		var items = ResNodes.getSelectors();
		var firstItem = null;
		for (var i = 0; i < items.length; i++) {
			var node = items.items(i);
			if (!firstItem) firstItem = (node == this.nodeFrom) ? this.nodeFrom : null;
			if (!firstItem) firstItem = (node == nodeTo)        ? nodeTo        : null;
			if (firstItem) node.className = "resSelected";
			if (firstItem != node) {
				if ((node == this.nodeFrom) || (node == nodeTo)) {
					break;
				}
			}
		}
	},
	/** mouesdown イベントを処理します。
	 * @param {event} e イベント
	 */
	onMouseDown: function(e) {
		var node = e.target;
		this.isButtonDown = (this.isEventInRange(e) && (e.button == 0)) ? true : false;
		if (this.isButtonDown) {
			if (!(e.ctrlKey || e.shiftKey)) if (node.parentNode.className != "resSelected") this.clear();
		 	e.preventDefault();
		 	this.nodeFrom = node.parentNode;
		 	this.modeSelect = (node.parentNode.className == "resSelected") ? false : true;
		 	this.onMouseOver(e);
		} else {
			if (e.button == 0) this.clear();
		}
	},
	/** mouseup イベントを処理します。
	 * @param {event} e イベント
	 */
	onMouseUp: function(e) {
		this.isButtonDown = false;
	},
	/** mouseover イベントを処理します。
	 * @param {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		if (this.isButtonDown && (node.className == "resContainer")) {
			var parentNode = node.parentNode;
			//this.selectNodes(parentNode);
			parentNode.className = this.modeSelect ? "resSelected" : "resUnselected";
			this.ensureVisible(node);
		}
	},
	/** コンテキストメニューの項目
	 * @type Array
	 */
	items: ([
		["<b>返信...</b>", "reply"],
		["-"],
		[["コピー",SKIN_PATH + "img/copy.png"],[
			["すべて", "copyAll"],
			["すべて(Jane 形式)", "copyAllJane"],
			["-"],
			["本文",   "copyBody"],
			["名前",   "copyName"],
			["メール", "copyMail"],
			["日付",   "copyDate"],
			["ID",     "copyID"],
			["BeID",   "copyBeID"],
			["-"],
			["このレスのURL",    "copyUrl"]
		]]
		]),
	/** ContextMenu オブジェクト
	 * @type ContextMenu
	 */
	contextMenu: null,
	/** contextmenu イベントを処理します。
	 * @param {event} e イベント
	 */
	onContextMenu: function(e) {
		var node = e.target;
		if (this.isEventInRange(e)) {
			if (this.length > 0) {
				this.contextMenu = new ContextMenu(this.items);
				this.contextMenu.onClick = this.onMenuItemClick.bind(this);
				this.contextMenu.show(e.clientX, e.clientY);
				e.preventDefault();
				e.stopPropagation();
			}
		}
	},
	/** メニュー項目がクリックされたときの処理をします。
	 * @param {String} caption メニュー項目のキャプション
	 * @param {String} id      メニュー項目の ID
	 */
	onMenuItemClick: function(caption, id) {
		switch (id) {
			case "reply":
				ThreadDocument.writeTo(this.getAnchorString());
				break;
			case "copyAll":
			case "copyAllJane":
				var items = ResNodes.getSelectors().selected;
				if (!items.length) return;
				var format = (id == "copyAllJane") ? "Jane" : "2ch";
				var buf = [];
				for (var i = 0; i < items.length; i++) {
					var node = items.items(i);
					buf.push(ResNodes.getEntireTextByIndex(ResNodes.getIndexByContainer(node.firstChild), format));
				}
				ThreadDocument.setClipboard(buf.join("\n\n"));
				break;
			case "copyBody":
			case "copyName":
			case "copyMail":
			case "copyDate":
			case "copyID":
			case "copyBeID":
				var f = {
					"copyBody": ResNodes.getBodyText.bind(ResNodes),
					"copyName": ResNodes.getNameText.bind(ResNodes),
					"copyMail": ResNodes.getMailText.bind(ResNodes),
					"copyDate": ResNodes.getDateText.bind(ResNodes),
					"copyID":   ResNodes.getIDText.bind(ResNodes),
					"copyBeID": ResNodes.getBeIDText.bind(ResNodes) };
				var items = ResNodes.getSelectors().selected;
				if (!items.length) return;
				var buf = [];
				for (var i = 0; i < items.length; i++) {
					var node = items.items(i);
					buf.push(f[id](node));
				}
				ThreadDocument.setClipboard(buf.join((id == "copyBody") ? "\n\n" : "\n"));
				break;
			case "copyUrl":
				var numbers = this.getAnchorString().split(",");
				for (var i = 0; i < numbers.length; i++) {
					numbers[i] = EXACT_URL + numbers[i];
				}
				ThreadDocument.setClipboard(numbers.join("\n") + "\n");
				break;
		}
	}
}

/**
 * Firefox の検索でヘッダが隠れる問題に対する対処を行います。
 * @static
 */
var FxFindHandler = {
	/** ドラッグ中かどうか
	 * @type Boolean
	 */
	isDragging: false,
	/** Shift キーが押されているかどうか
	 * @type Boolean
	 */
	isShiftKeyPressed: false,
	/** 前回の選択範囲
	 * @type Range
	 */
	prevRange: null,
	/** 要素の位置とサイズを取得します。
	 * @param  {element} src 要素
	 * @return {object} left, top, width, height, right, bottom の各メンバから成るオブジェクト
	 */
	getRect: function(src) {
		var parent = src;
		var x = 0;
		var y = 0;
		while (parent) {
			x += parent.offsetLeft;
			y += parent.offsetTop;
			parent = parent.offsetParent;
		}
		return {left: x, top: y, width: src.offsetWidth, height: src.offsetHeight,
		        right: x + src.offsetWidth, bottom: y + src.offsetHeight};
	},
	/** 要素の表示を強制します。
	 * @param {element} node 要素
	 */
	ensureVisible: function(node) {
		var rc = this.getRect(node);
		if (rc.top < (window.scrollY + Nodes.header.offsetTop + Nodes.header.offsetHeight)) {
			window.scrollTo(0, rc.top - (Nodes.header.offsetTop + Nodes.header.offsetHeight));
		}
	},
	/** 選択範囲を保存します。*/
	preventScroll: function() {
		var selection = window.getSelection();
		if (!selection.rangeCount) return;
		var range = selection.getRangeAt(0);
		if (range.collapsed) return;
		this.prevRange = range;
	},
	/** イベントリスナを登録します。*/
	initialise: function() {
		window.addEventListener("mousedown", (function(e) { this.isDragging = (e.button == 0) }).bind(this), false);
		window.addEventListener("mouseup",   (function(e) { 
			this.isDragging = false;
			this.preventScroll();
			}).bind(this), false);
		window.addEventListener("keypress",  (function(e) { 
			this.isShiftKeyPressed = e.shiftKey;
			this.preventScroll();
			}).bind(this), false);
		window.setInterval(this.onTimer.bind(this), 10);
	},
	/** タイマーでマウスやキーイベントを伴わない選択範囲の変化を監視します。 */
	onTimer: function() {
		if (this.isDragging || this.isShiftKeyPressed) return;
		var selection = window.getSelection();
		if (!selection.rangeCount) return;
		var range = selection.getRangeAt(0);
		if (range.collapsed) return;
		if (range == this.prevRange) return;
		// text nodes have no positions
		var node = (range.startContainer.nodeType == 3) ? range.startContainer.parentNode : range.startContainer;
		this.ensureVisible(node);
		this.prevRange = range;
	}
}

/**
 * 数字キーで該当レスへスクロールする機能を管理します。
 * @static
 */
var NumericScroller = {
	/** 入力バッファ
	 * @type String
	 */
	number: "",
	/** 前回のキー入力時の時間
	 * @type Number
	 */
	time: 0,
	/** 入力バッファをクリアします。*/
	clear: function() {
		this.number = "";
	},
	/** keypress イベントを処理します。
	 * @param {event} e イベント
	 */
	onKeyPress: function(e) {
		if (e.target == Nodes.findBoxText) return;
		if ((e.which < 48) || (e.which > 57)) {
			this.clear();
		} else {
			if (parseInt(this.number) > ThreadDocument.countAll) this.clear();
			if ((new Date().getTime() - this.time) > 1000) this.clear(); // 1000 ms
			this.number += String.fromCharCode(e.which);
			var node = ResNodes.getContainerByIndex(this.number);
			if (node) window.scrollTo(0, node.offsetTop - (Nodes.header.offsetTop + Nodes.header.offsetHeight));
		}
		this.time = new Date().getTime();
	},
	/** イベントリスナを追加します。*/
	initialise: function() {
		window.addEventListener("keypress",  this.onKeyPress.bind(this), false);
	}
}

/**
 * 不用意なポップアップを禁止する機能を管理します。
 * @static
 */
var PopupPreventer = {
	/** スクロールされたかどうか
	 * @type Boolean
	 */
	isScrolled: false,
	/** 最後にスクロールされたときに時間
	 * @type Number
	 */
	lastScrolledTime: null,
	/** タイマー ID
	 * @type Number
	 */
	timerID: null,
	/** イベントリスナを登録します。*/
	initialise: function() {
		window.addEventListener("mouseover", this.onMouseOver.bind(this), true);
		window.addEventListener("scroll", (function() {
			this.isScrolled = true;
			this.lastScrolledTime = new Date().getTime();
		}).bind(this), true);
		window.addEventListener("mousemove", (function() {
			if ((new Date().getTime() - this.lastScrolledTime) > 250) this.isScrolled = false;
		}).bind(this), true);
	},
	/** mouseover イベントを処理します。
	 * @param {event} e イベント
	 */
	onMouseOver: function(e) {
		if (this.isScrolled) {
			e.preventDefault();
			e.stopPropagation();
		}
	}
}