﻿// option.js
// (2007/02/02)
// オプションダイアログを動的に作成/管理する
// pref.jsと連携するようにできてるので勝手にマージ

/**
 * @fileOverview オプション関係を扱います。
 */

/**
 * globalStorage にアクセスします。デフォルトスキンの pref.js がベースになっています。
 * @static
 */
var SkinPref = {
	/** スキンの名称
	 * @type String
	 */
	_skinName: "smorgasbord",
	/** スキンのバージョン
	 * @type String
	 */
	_skinVersion: "2007/09/25 afternoon (still unstable)",
	/** URI
	 * @type String
	 */
	_skinURI: "http://smorgasbord.drwatson.nobody.jp/",
	/** メールアドレス
	 * @type String
	 */
	_skinMail: "\u0064\u0072\u0077\u0061\u0074\u0073\u006f\u006e\u002e\u0065\u0078\u0065\u002b\u0073\u006d\u006f\u0072\u0067\u0061\u0073\u0062\u006f\u0072\u0064\u0040\u0067\u006d\u0061\u0069\u006c\u002e\u0063\u006f\u006d",
	/** globalStorage
	 * @type StorageList
	 */
	_storage: (location.href.indexOf("http://localhost", 0) == -1) ? globalStorage["127.0.0.1"] : globalStorage["localhost.localdomain"],
	/** スキン名とキーの名前を結合して、名前を解決します。
	 * @param  {String} aPrefName キー
	 * @return {String} 結果
	 */
	_resolvePrefName: function(aPrefName){
		return this._skinName + "_" + aPrefName;
	},
	/** globalStorage から文字列を取得します。
	 * @param  {String} aPrefName     キー
	 * @return {String} aDefaultValue デフォルト値
	 * @return {String} 値
	 */
	getStr: function(aPrefName, aDefaultValue){
		var item = this._storage.getItem(this._resolvePrefName(aPrefName));
		if(item == null) return aDefaultValue || "";
		return item.value;
	},
	/** globalStorage に文字列を設定します。
	 * @param  {String} aPrefName キー
	 * @return {String} aValue    値
	 * @return {String} 値
	 */
	setStr: function(aPrefName, aValue){
		var value = String(aValue);
		if (value != this.getStr(aPrefName))
			this._storage.setItem(this._resolvePrefName(aPrefName), value);
		return value;
	},
	/** globalStorage から数値を取得します。
	 * @param  {String} aPrefName     キー
	 * @return {Number} aDefaultValue デフォルト値
	 * @return {Number} 値
	 */
	getInt: function(aPrefName, aDefaultValue){
		var item = this._storage.getItem(this._resolvePrefName(aPrefName));
		if(item == null) return aDefaultValue || 0;
		return parseInt(item.value);
	},
	/** globalStorage に数値を設定します。
	 * @param  {String} aPrefName キー
	 * @return {Number} aValue    値
	 * @return {Number} 値
	 */
	setInt: function(aPrefName, aValue){
		var value = parseInt(aValue);
		if (value != this.getInt(aPrefName))
			this._storage.setItem(this._resolvePrefName(aPrefName), value);
		return value;
	},
	/** globalStorage からブール値を取得します。
	 * @param  {String}  aPrefName     キー
	 * @return {Boolean} aDefaultValue デフォルト値
	 * @return {Boolean} 値
	 */
	getBool: function(aPrefName, aDefaultValue){
		var item = this._storage.getItem(this._resolvePrefName(aPrefName));
		if(item == null) return aDefaultValue || false;
		return (item.value == "true");
	},
	/** globalStorage からブール値を取得します。
	 * @param  {String}  aPrefName キー
	 * @return {Boolean} aValue    値
	 * @return {Boolean} 値
	 */
	setBool: function(aPrefName, aValue){
		var value = (aValue) ? "true" : "false";
		if (value != String(this.getBool(aPrefName)))
			this._storage.setItem(this._resolvePrefName(aPrefName), value);
		return value;
	},
	/** globalStorage からキーを削除します。
	 * @param  {String}  aPrefName キー
	 */
	remove: function(aPrefName){
		this._storage.removeItem(this._resolvePrefName(aPrefName));
	}

};

/**
 * [オプション] ダイアログを管理します。
 * @static
 */
var dialogOptions = {
	/** ダイアログの親要素
	 * @type element
	 */
	content: document.createElement("div"),
	/** [OK] ボタンが押されたときの処理をします。*/
	onOK: function(){
		var items = document.getElementsByTagName("label");
		for (var i = 0; i < items.length; i++) {
			if (items[i].className == "dialogOptionsLabel") {
				if (items[i].firstChild.tagName == "INPUT") {
					switch (items[i].firstChild.getAttribute("type")) {
					case "checkbox":
						SkinPref.setBool(items[i].firstChild.id, items[i].firstChild.checked);
						break;
					case "radio":
						SkinPref.setBool(items[i].firstChild.id, items[i].firstChild.checked);
					}
				} else if (items[i].lastChild.tagName == "INPUT") {
					if (items[i].lastChild.getAttribute("type") == "text") {
						SkinPref.setStr(items[i].lastChild.id, items[i].lastChild.value);
					}
				} else if (items[i].lastChild.tagName == "SELECT") {
					SkinPref.setInt(items[i].lastChild.id, items[i].lastChild.selectedIndex);
				}
			}
		}
		this.hide();
	},
	/** [キャンセル] ボタンが押されたときの処理をします。*/
	onCancel: function(){
		this.hide();
	},
	/** リサイズされたときの処理をします。*/
	onResize: function(){
		dialogOptions.content.style.left = (document.body.offsetWidth - dialogOptions.content.offsetWidth) / 2
		dialogOptions.content.style.top  = (window.innerHeight - dialogOptions.content.offsetHeight) / 2
		var bg = document.getElementById("dialogOptionsBackground");
		if (bg) {
			bg.style.width = document.body.offsetWidth;
			bg.style.height = window.innerHeight;
		}
	},
	/** ダイアログを表示します。*/
	show: function(){
		var bg = document.createElement("div");
		bg.id = "dialogOptionsBackground";
		bg.style.width = document.body.offsetWidth;
		bg.style.height = window.innerHeight;
		document.body.appendChild(bg);

		this.content.style.position   = "fixed";
		this.content.style.width      = 512;
		this.content.style.visibility = "hidden"
		this.content.style.display    = "block";
		this.selectPage(this.content.childNodes[1].id.replace("Page_", ""));
		this.content.style.left = (document.body.offsetWidth - this.content.offsetWidth) / 2
		this.content.style.top  = (window.innerHeight - this.content.offsetHeight) / 2
		this.content.style.visibility = "visible";
		window.addEventListener("resize", dialogOptions.onResize, false);
	},
	/** ダイアログを非表示にします。*/
	hide: function(){
		this.content.style.display = "none";
		var bg = document.getElementById("dialogOptionsBackground");
		if (bg) document.body.removeChild(bg);
		window.removeEventListener("resize", dialogOptions.onResize, false);
	},
	/** ダイアログのページを切り替えます。
	 * @param {String} id ページの ID
	 */
	selectPage: function(id){
		var divHeader = document.getElementById("dialogOptionsHeader");
		for (var i = 0; i < divHeader.childNodes.length - 1; i++) {
			var icon = divHeader.childNodes[i];
			if (icon.id == "Icon_" + id) {
				icon.className = "icon" + " " + "selected";
			} else {
				icon.className = "icon";
			}
		}	
		for (var i = 1; i < this.content.childNodes.length - 1; i++) {
			var page = this.content.childNodes[i];
			if (page.id == "Page_" + id) {
				page.style.display = "block";
			} else {
				page.style.display = "none";
			}
		}
		if ((this.content.offsetTop + this.content.offsetHeight) > window.innerHeight) this.onResize();
	},
	/** ページを追加します。
	 * @param {String} id      ID
	 * @param {String} caption キャプション
	 * @param {String} icon    アイコンの URI
	 */
	addPage: function(id, caption, icon){
		if (!this.content.id) this.content.id = "dialogOptions";
		if (!document.getElementById("dialogOptions")) document.body.appendChild(this.content);
		var divHeader = document.getElementById("dialogOptionsHeader");
		if (!divHeader) {
			divHeader = document.createElement("div");
			divHeader.id = "dialogOptionsHeader";
			divHeader.className = "header";
			var divHeaderEnd = document.createElement("div");
			divHeaderEnd.id = "dialogOptionsHeaderEnd";
			divHeaderEnd.className = "null";
			divHeader.appendChild(divHeaderEnd);
			this.content.appendChild(divHeader);
			
			var p      = document.createElement("p");
			var ok     = document.createElement("input");
			ok.setAttribute("type",  "submit");
			ok.setAttribute("value", "OK");
			ok.setAttribute("onclick", "dialogOptions.onOK()");
			var cancel = document.createElement("input");
			cancel.setAttribute("type",  "button");
			cancel.setAttribute("value", "キャンセル");
			cancel.setAttribute("onclick", "dialogOptions.onCancel()");
			p.appendChild(ok);
			p.appendChild(cancel);
			this.content.appendChild(p);
		}
		
		var divIcon = document.createElement("div");
		divIcon.id = "Icon_" + id;
		divIcon.className = "icon";
		divIcon.textContent = caption;
		divIcon.style.backgroundImage = "url(" + icon + ")";
		divIcon.setAttribute("onclick", "dialogOptions.selectPage('" + id + "')");
		divHeader.insertBefore(divIcon, document.getElementById("dialogOptionsHeaderEnd"));
		
		var divPage = document.createElement("div");
		divPage.id = "Page_" + id;
		divPage.style.display = "none";
		this.content.insertBefore(divPage, this.content.lastChild);
	},
	/** フレームを追加します。
	 * @param {String} page      追加するページの ID
	 * @param {String} id        ID
	 * @param {String} caption   キャプション
	 */
	addFrame: function(page, id, caption){
		var fieldset = document.createElement("fieldset");
		fieldset.id = "Frame_" + page + "_" + id;
		var legend  = document.createElement("legend");
		legend.textContent = caption;
		fieldset.appendChild(legend);
		fieldset.appendChild(document.createElement("ul"));
		var divPage = document.getElementById("Page_" + page);
		if (divPage) divPage.appendChild(fieldset);
	},
	/** ラベルを追加します。
	 * @param {String} page      追加するページの ID
	 * @param {String} frame     フレームの ID
	 * @param {String} id        ID
	 * @param {String} caption   キャプション
	 * @param {String} [level]   インデントする階層
	 */
	addLabel: function(page, frame, id, caption, level){
		var parentObject = document.getElementById("Frame_" + page + "_" + frame).lastChild;
		if (level > 0) {
			for (var i = 1; i <= level; i++) {
				parentObject.appendChild(document.createElement("ul"));
				parentObject = parentObject.lastChild;
			}
		}
		var li = document.createElement("li");
		var label = document.createElement("label");
		label.className = "dialogOptionsLabel";
		var span = document.createElement("span");
		span.textContent = caption;
		label.appendChild(span);
		li.appendChild(label);
		parentObject.appendChild(li);
	},
	/** リンクを追加します。
	 * @param {String} page      追加するページの ID
	 * @param {String} frame     フレームの ID
	 * @param {String} id        ID
	 * @param {String} caption   キャプション
	 * @param {String} uri   　　URI
	 * @param {Number} [level]   インデントする階層
	 */
	addLink: function(page, frame, id, caption, uri, level){
		var parentObject = document.getElementById("Frame_" + page + "_" + frame).lastChild;
		if (level > 0) {
			for (var i = 1; i <= level; i++) {
				parentObject.appendChild(document.createElement("ul"));
				parentObject = parentObject.lastChild;
			}
		}
		var li = document.createElement("li");
		var label = document.createElement("label");
		label.className = "dialogOptionsLabel";
		var a = document.createElement("a");
		a.setAttribute("href", uri);
		a.setAttribute("target", "_blank");
		a.textContent = caption;
		label.appendChild(a);
		li.appendChild(label);
		parentObject.appendChild(li);
	},
	/** チェックボックスを追加します。
	 * @param {String}  page         追加するページの ID
	 * @param {String}  frame        フレームの ID
	 * @param {String}  id           ID
	 * @param {String}  caption      キャプション
	 * @param {Boolean} defaultValue デフォルト値
	 * @param {Number}  [level]      インデントする階層
	 */
	addCheckBox: function(page, frame, id, caption, defaultValue, level){
		var parentObject = document.getElementById("Frame_" + page + "_" + frame).lastChild;
		if (level > 0) {
			for (var i = 1; i <= level; i++) {
				parentObject.appendChild(document.createElement("ul"));
				parentObject = parentObject.lastChild;
			}
		}
		var li = document.createElement("li");
		var label = document.createElement("label");
		label.className = "dialogOptionsLabel";
		var input = document.createElement("input");
		input.setAttribute("type", "checkbox");
		input.id = id;
		input.checked = SkinPref.getBool(id, defaultValue);
		label.appendChild(input);
		var span = document.createElement("span");
		span.textContent = caption;
		label.appendChild(span);
		li.appendChild(label);
		parentObject.appendChild(li);
	},
	/** ラジオボタンを追加します。
	 * @param {String}  page         追加するページの ID
	 * @param {String}  frame        フレームの ID
	 * @param {String}  id           ID
	 * @param {String}  caption      キャプション
	 * @param {Boolean} defaultValue デフォルト値
	 * @param {Number}  [level]      インデントする階層
	 */
	addRadio: function(page, frame, id, caption, defaultValue, level){
		var parentObject = document.getElementById("Frame_" + page + "_" + frame).lastChild;
		if (level > 0) {
			for (var i = 1; i <= level; i++) {
				parentObject.appendChild(document.createElement("ul"));
				parentObject = parentObject.lastChild;
			}
		}
		var li = document.createElement("li");
		var label = document.createElement("label");
		label.className = "dialogOptionsLabel";
		var input = document.createElement("input");
		input.setAttribute("type", "radio");
		input.setAttribute("name", frame);
		input.id = id;
		input.checked = SkinPref.getBool(id, defaultValue);
		label.appendChild(input);
		var span = document.createElement("span");
		span.textContent = caption;
		label.appendChild(span);
		li.appendChild(label);
		parentObject.appendChild(li);
	},
	/** エディットボックスを追加します。
	 * @param {String} page         追加するページの ID
	 * @param {String} frame        フレームの ID
	 * @param {String} id           ID
	 * @param {String} caption      キャプション
	 * @param {String} defaultValue デフォルト値
	 * @param {Number} [size]       サイズ
	 * @param {Number} [level]      インデントする階層
	 */
	addEditBox: function(page, frame, id, caption, defaultValue, size, level){
		var parentObject = document.getElementById("Frame_" + page + "_" + frame).lastChild;
		if (level > 0) {
			for (var i = 1; i <= level; i++) {
				parentObject.appendChild(document.createElement("ul"));
				parentObject = parentObject.lastChild;
			}
		}
		var li = document.createElement("li");
		var label = document.createElement("label");
		label.className = "dialogOptionsLabel";
		var span = document.createElement("span");
		span.textContent = caption;
		label.appendChild(span);
		var input = document.createElement("input");
		input.setAttribute("type", "text");
		input.value = SkinPref.getStr(id, defaultValue);
		if (size) {
			input.style.width = size + "em";
		}
		input.id = id;
		
		label.appendChild(input);
		li.appendChild(label);
		parentObject.appendChild(li);
	},
	/** 数値のみを入力できるエディットボックスを追加します。
	 * @param {String} page         追加するページの ID
	 * @param {String} frame        フレームの ID
	 * @param {String} id           ID
	 * @param {String} caption      キャプション
	 * @param {Number} defaultValue デフォルト値
	 * @param {Number} [size]       サイズ
	 * @param {Number} [level]      インデントする階層
	 */
	addNumberBox: function(page, frame, id, caption, defaultValue, size, level){
		var parentObject = document.getElementById("Frame_" + page + "_" + frame).lastChild;
		if (level > 0) {
			for (var i = 1; i <= level; i++) {
				parentObject.appendChild(document.createElement("ul"));
				parentObject = parentObject.lastChild;
			}
		}
		var li = document.createElement("li");
		var label = document.createElement("label");
		label.className = "dialogOptionsLabel";
		var span = document.createElement("span");
		span.textContent = caption;
		label.appendChild(span);
		var input = document.createElement("input");
		input.setAttribute("type", "text");
		input.style.textAlign = "right";
		if (size) {
			input.style.width = size + "em";
		}
		input.id = id;
		input.value = SkinPref.getStr(id, defaultValue);
		input.setAttribute("onkeydown", "var i = event.keyCode; if ((i >= 96 && i <= 105) || (i >= 48 && i <= 57) || (i == 190) | (i == 8) || (i == 35) || (i == 36) ||(i == 37) || (i == 39) || (i == 46)) { return true; } return false;");
		input.setAttribute("onkeyup",  "if (event.target.value.match(/[^0-9\.]+/)) event.target.value='';");
		input.setAttribute("onfocus",  "if (event.target.value.match(/[^0-9\.]+/)) event.target.value='';");
		input.setAttribute("onblur",  "if (event.target.value.match(/[^0-9\.]+/)) event.target.value='';");
		label.appendChild(input);
		li.appendChild(label);
		parentObject.appendChild(li);
	},
	/** コンボボックスを追加します。
	 * @param {String} page         追加するページの ID
	 * @param {String} frame        フレームの ID
	 * @param {String} id           ID
	 * @param {String} caption      キャプション
	 * @param {Array}  value        項目
	 * @param {Number} defaultValue デフォルト値
	 * @param {Number} [size]       サイズ
	 * @param {Number} [level]      インデントする階層
	 */
	addComboBox: function(page, frame, id, caption, value, defaultValue, level){
		var parentObject = document.getElementById("Frame_" + page + "_" + frame).lastChild;
		if (level > 0) {
			for (var i = 1; i <= level; i++) {
				parentObject.appendChild(document.createElement("ul"));
				parentObject = parentObject.lastChild;
			}
		}
		var li = document.createElement("li");
		var label = document.createElement("label");
		label.className = "dialogOptionsLabel";
		var span = document.createElement("span");
		span.textContent = caption;
		label.appendChild(span);
		var select = document.createElement("select");
		select.id = id;
		for (var i = 0; i < value.length; i++) {
			var option = document.createElement("option");
			option.textContent = value[i];
			option.setAttribute("value", value[i]);
			select.appendChild(option);
		}
		select.selectedIndex = SkinPref.getInt(id, defaultValue);
		label.appendChild(select);
		li.appendChild(label);
		parentObject.appendChild(li);
	}
};

/** [オプション] ダイアログを初期化します。*/
function initOptionDialog(){
	dialogOptions.addPage     ("General", "一般",  SKIN_PATH + "img/general.png");
	dialogOptions.addFrame    ("General", "Popup", "ポップアップ");
	//dialogOptions.addLabel    ("General", "Popup", "labelPopup", "次のポップアップを有効にする:");
	dialogOptions.addCheckBox ("General", "Popup", "enableResPopup", "アンカー", true);
	dialogOptions.addCheckBox ("General", "Popup", "enableIDPopup", "ID", true);
	dialogOptions.addCheckBox ("General", "Popup", "enableIDPopupOnClick", "クリックしてポップアップ", false, 1);
	dialogOptions.addCheckBox ("General", "Popup", "enableIDPopupAll", "すべてのレスを一度に表示する", false, 1);
	dialogOptions.addCheckBox ("General", "Popup", "enableImagePopup", "画像", true);
	dialogOptions.addComboBox ("General", "Popup", "valuePixelationMethod",   "フィルタ: ", ["モザイク", "ガウスぼかし(Fx3)"], 0, 1);
	dialogOptions.addComboBox ("General", "Popup", "valuePixelationSize",     "レベル: ", ["強", "中", "弱"], 1, 1);
	dialogOptions.addCheckBox ("General", "Popup", "enableDefaultPixelation", "デフォルトでぼかす", false, 1);
	dialogOptions.addCheckBox ("General", "Popup", "enablePrecisePixelation", "正確にモザイク化する(遅い)", false, 1);
	dialogOptions.addCheckBox ("General", "Popup", "enableUrlPopup", "リンク先サムネイル", true);
	dialogOptions.addComboBox ("General", "Popup", "valueUrlPopupSize", "サイズ: ", ["大", "小"], 0, 1);
	dialogOptions.addCheckBox ("General", "Popup", "enableVideoPopup", "動画", true);
	dialogOptions.addCheckBox ("General", "Popup", "videoPopupAutoStart", "自動再生", true, 1);
	dialogOptions.addCheckBox ("General", "Popup", "enableTrackBackPopup", "逆参照", true);
	dialogOptions.addCheckBox ("General", "Popup", "enableTrackBackPopupAll", "すべてのレスを一度に表示する", false, 1);
	
	dialogOptions.addPage     ("Special", "詳細",  SKIN_PATH + "img/special.png");
	dialogOptions.addFrame    ("Special", "Link",  "リンク");
	dialogOptions.addCheckBox ("Special", "Link",  "enableNewWindow", "リンクを新しいウィンドウで開く", true);
	dialogOptions.addCheckBox ("Special", "Link",  "enableNoReferer", "リファラを送信しない", true);
	
	dialogOptions.addFrame    ("Special", "Visualisation",  "視覚効果");
	dialogOptions.addCheckBox ("Special", "Visualisation",  "enableContract", "スレッドタイトルを表示幅に合わせて省略表示する", true);
	dialogOptions.addCheckBox ("Special", "Visualisation",  "enablePopupFade", "ポップアップのフェードアウトを有効にする", true);
	dialogOptions.addComboBox ("Special", "Visualisation",  "valuePopupFadeStep", "フェードのステップ: ", new Array(0.1,0.2,0.25,0.33,0.5), 1, 1);
	dialogOptions.addCheckBox ("Special", "Visualisation",  "enableFindHighlight", "検索/抽出時に強調表示を行う", false);

	dialogOptions.addFrame    ("Special", "Browse", "ブラウズ");
	dialogOptions.addNumberBox("Special", "Browse", "valueHistoryItemCounts", "保存する履歴の件数: ", 32, 3);
	dialogOptions.addCheckBox ("Special", "Browse", "enableHookReload", "F5 キーをフックして動的に更新を行う", true);
	dialogOptions.addCheckBox ("Special", "Browse", "enableSmoothScroll", "スムーズスクロールを有効にする", true);
	dialogOptions.addComboBox ("Special", "Browse", "valueSmoothScrollFrames", "フレーム数: ", new Array("最小","中","最大"), 1, 1);
	
	dialogOptions.addFrame    ("Special", "AutoPilot",  "オートパイロット");
	dialogOptions.addCheckBox ("Special", "AutoPilot", "enableAutoReloadOnLiveThread", "実況板では自動更新をデフォルトにする", false);
	dialogOptions.addCheckBox ("Special", "AutoPilot", "enableAutoReloadWhenInactive", "非アクティヴ時でも自動更新を強制する", false);
	dialogOptions.addComboBox ("Special", "AutoPilot", "valueAutoReloadInterval", "自動更新の間隔: ", new Array("15 秒","30 秒","60 秒"), 1);
	
	dialogOptions.addPage     ("About", "スキン情報",  "chrome://bbs2chreader/content/icon.png");
	dialogOptions.addFrame    ("About", "Info",  "このスキンについて");
	if (SkinPref._skinName)
		dialogOptions.addLabel("About", "Info", "labelSkinName", SkinPref._skinName);
	if (SkinPref._skinVersion)
		dialogOptions.addLabel("About", "Info", "labelSkinVersion", "バージョン: " + SkinPref._skinVersion);
	if (SkinPref._skinURI) {
		// dialogOptions.addLabel("About", "Info", "labelSkinBlank", "　");
		dialogOptions.addLink ("About", "Info", "labelSkinURI", SkinPref._skinURI, SkinPref._skinURI);
	}
	if (SkinPref._skinMail)
		dialogOptions.addLink ("About", "Info", "labelSkinMail", SkinPref._skinMail, "mailto:" + SkinPref._skinMail);	
	
	dialogOptions.addFrame    ("About", "Keymap",  "キーボードショートカット");
	dialogOptions.addLabel    ("About", "Keymap", "", "F5: 最新の情報に更新");
	dialogOptions.addLabel    ("About", "Keymap", "", "←: 前のレスへ移動");
	dialogOptions.addLabel    ("About", "Keymap", "", "→: 次のレスへ移動");
	dialogOptions.addLabel    ("About", "Keymap", "", "Ctrl + Enter: 書き込む");
	dialogOptions.addLabel    ("About", "Keymap", "", "Ctrl + Space: すべて表示");
	dialogOptions.addLabel    ("About", "Keymap", "", "Ctrl + Alt + F: 検索/抽出");
	dialogOptions.addLabel    ("About", "Keymap", "", "数字キー: 入力されたレス番号へスクロール");
}
window.addEventListener("load", initOptionDialog, false);
